package uap.bs.bap.fs.service;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import nc.bs.pub.bap.toolkit.AppExecutor;
import nc.jdbc.framework.mapping.IMappingMeta;
import nc.util.iufo.pub.OIDMaker;
import nc.vo.pub.lang.UFDateTime;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;

import uap.bs.bap.fs.storage.FileBodyStorageDAO;
import uap.bs.bap.fs.storage.FileExtPropStorageDAO;
import uap.bs.bap.fs.storage.FileStorageDAO;
import uap.pub.bap.fs.FileStorageUtil;
import uap.pub.bap.fs.domain.FileBody;
import uap.pub.bap.fs.domain.FileHeader;
import uap.pub.bap.fs.domain.FileStorageConfigFactory;
import uap.pub.bap.fs.domain.FileStorageException;
import uap.pub.bap.fs.domain.IFileStorageConst;
import uap.pub.bap.fs.domain.ext.FileStorageExtPackage;
import uap.pub.bap.fs.domain.ext.FileStorageMappingMeta;
import uap.pub.bap.fs.domain.ext.FileStoreExtInfo;
import uap.pub.bap.fs.domain.ext.IFileStorageExt;

import com.ufida.iufo.pub.tools.AppDebug;

public class FileMetaStorageService implements IFileMetaStorageService {

	/** ļͷɾĲDAO */
	private FileStorageDAO fileHeaderDao;

	/** ļɾĲDAO */
	private FileBodyStorageDAO fileBodyDaO;

	/** չDAO */
	private FileExtPropStorageDAO fileExtPropDao;

	private static Lock l = new ReentrantLock();

	@Override
	public FileHeader updateMetaData(String moduleId, String fileName,
			String fileType, FileHeader fileHeader, int fileVersion,
			String storeDir, String relativePath, File file, boolean override,
			String currUser, IFileStorageExt... fileStorageExt) {
		FileHeader updateHeader = null;
		try {
			// ļͷ
			String storePath = storeDir + file.getName();
			long fileSize = file.length();
			fileHeader = updateFileHeader(moduleId, fileName, relativePath,
					fileHeader, fileVersion, override, fileSize, currUser);
			// ļ
			updateFileBody(fileType, fileHeader, storePath, fileSize);
			// չ
			updateHeader = updateExtVos(override, fileHeader, currUser,
					fileStorageExt);

		} catch (Exception e) {
			AppDebug.error("ȡļԪϢ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0340") /*
																			 * @res
																			 * *
																			 * "ȡļԪϢ"
																			 */
					+ e.getMessage());
		} finally {
			AppExecutor.postProcess();
		}
		return updateHeader;
	}

	@Override
	public FileHeader updateEmptyMetaData(String moduleId, String fileName,
			String relativePath, String storeDir, FileHeader fileHeader,
			int fileVersion, boolean override, String currUser,
			IFileStorageExt... fileStorageExt) {
		FileHeader updateHeader = null;
		try {
			fileHeader = updateFileHeader(moduleId, fileName, relativePath,
					fileHeader, fileVersion, override, 0L, currUser);
			// ļ
			String storePath = storeDir + fileName;
			updateFileBody(FileStorageUtil.getFileType(fileName), fileHeader,
					storePath, 0L);
			// չ
			updateHeader = updateExtVos(override, fileHeader, currUser,
					fileStorageExt);
		} catch (Exception e) {
			AppDebug.error("ȡļԪϢ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0340") /*
																			 * @res
																			 * *
																			 * "ȡļԪϢ"
																			 */
					+ e.getMessage());
		} finally {
			AppExecutor.postProcess();
		}
		return updateHeader;
	}

	@Override
	public FileHeader updateFileHeader(String moduleId, String fileName,
			String relativePath, FileHeader fileHeader, int fileVersion,
			boolean override, long fileSize, String currUser) throws Exception {
		currUser = FileStorageUtil.getCurrentUser(currUser);
		if (!override) {
			if (fileHeader != null) {// fileHeaderΪգ˵Ѿ˼¼ʱ汾һλ
				fileVersion++;
			}
			fileHeader = null;// ǣ൱½һļҪݿ¼һ¼
		}
		// ļͷ
		FileHeader header = null;
//		PerfWatch pw = new PerfWatch("дļmoduleId=" + moduleId + " fileName="
//				+ fileName);
		try {
			if (fileHeader == null) {// 1.ļûļͷϢҪһµļͷϢ档
				fileHeader = new FileHeader();
				fileHeader.setCreatetime(new UFDateTime());
				fileHeader.setCreator(currUser);
				fileHeader.setModule(moduleId);
				fileHeader.setName(fileName);
				fileHeader.setLastModifytime(new UFDateTime());
				fileHeader.setLastModifier(currUser);
				if (displayByName()) {
					fileHeader.setPath(relativePath);
				} else {// ʾ
					// ݿбpub_oidű
					String idPath = OIDMaker.getGE(
							FileStorageConfigFactory.getConfig().getDsName())
							.generate();
					fileHeader.setPath(idPath);
				}
				fileHeader.setFileSize(fileSize);
				fileHeader.setFileVersion(fileVersion);
//				pw.appendMessage(fileHeader.toString());
				header = getFileHeaderDao().createFileHeader(fileHeader);//
			} else {// FileHeaderΪգ˵֮ǰм¼ִи²
				fileHeader.setLastModifier(currUser);
				fileHeader.setLastModifytime(new UFDateTime());
				fileHeader.setFileSize(fileSize);
				header = getFileHeaderDao().updateFileHeader(fileHeader);
			}
		} finally {
			AppExecutor.postProcess();
//			pw.stop();
		}

		return header;
	}

	@Override
	public void updateFileBody(String fileType, FileHeader fileHeader,
			String storePath, long fileSize) throws Exception {
		try {
			String headId = fileHeader.getGuid();
			FileBody fileBody = getFileBodyDAO().getFileBody(headId);// ȡfileBody
			if (fileBody == null) {
				fileBody = new FileBody();
				fileBody.setFiletype(fileType);
				fileBody.setHeadId(fileHeader.getGuid());
				fileBody.setStorePath(storePath);// ļ
				fileBody.setFileSize(fileSize);
				getFileBodyDAO().createFileBody(fileBody);
			} else {
				fileBody.setFiletype(fileType);
				fileBody.setStorePath(storePath);// ļ
				fileBody.setFileSize(fileSize);
				getFileBodyDAO().updateFileBody(fileBody);
			}
		} finally {
			AppExecutor.postProcess();
		}
	}

	@Override
	public FileHeader updateExtVos(boolean override, FileHeader fileHeader,
			String currUser, IFileStorageExt... fileStorageExt) {
		if (ArrayUtils.isEmpty(fileStorageExt)) {
			return fileHeader;
		}
		currUser = FileStorageUtil.getCurrentUser(currUser);
		FileHeader updateHeader = null;
		try {
			if (override) {
				deleteExtVos(fileHeader);
			}
			StringBuilder tableNames = new StringBuilder();
			StringBuilder extProps = new StringBuilder();
			StringBuilder extTypes = new StringBuilder();
			String headerId = fileHeader.getGuid();
			// жϱǷ
			l.lock();
			try {
				for (IFileStorageExt extVo : fileStorageExt) {
					IMappingMeta meta = new FileStorageMappingMeta(extVo);
					if (!getFileExtPropDao().isTableExisted(meta)) {// NOSONAR
						// ڣ򴴽һű
						if(extVo instanceof FileStorageExtPackage){
							getFileExtPropDao().createTable((FileStorageExtPackage)extVo);
						}else{
							getFileExtPropDao().createTable(meta);// NOSONAR
						}
						
					}
				}
			} finally {
				l.unlock();
			}
			
			FileStoreExtInfo[] extVoInfos = null;
			if(fileStorageExt instanceof FileStorageExtPackage[]){
				extVoInfos = getFileExtPropDao().insertObjectNew(
						headerId, (FileStorageExtPackage[])fileStorageExt);
			}else{
				extVoInfos = getFileExtPropDao().insertObject(
						headerId, fileStorageExt);
			}
			
			if (!ArrayUtils.isEmpty(extVoInfos)) {
				for (FileStoreExtInfo extVoInfo : extVoInfos) {
					extProps.append(extVoInfo.getExtVoPK()).append(
							FileStorageDAO.SEPERATOR);
					tableNames.append(extVoInfo.getExtVoTableName()).append(
							FileStorageDAO.SEPERATOR);
					extTypes.append(extVoInfo.getExtVoType()).append(
							FileStorageDAO.SEPERATOR);
				}
			}
			getFileHeaderDao().insertExtInfo(headerId, extProps.toString(),
					tableNames.toString(), extTypes.toString());
			fileHeader.setLastModifytime(new UFDateTime());
			fileHeader.setLastModifier(currUser);
			updateHeader = getFileHeaderDao().updateFileHeader(fileHeader);
		} catch (Exception e) {
			AppDebug.error("չϢ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0355") /*
																			 * @res
																			 * *
																			 * "չϢ"
																			 */
					+ e.getMessage());
		} finally {
			AppExecutor.postProcess();
		}
		return updateHeader;
	}

	/**
	 * ɾչϢ
	 * 
	 * @param fileHeader
	 */
	private void deleteExtVos(FileHeader fileHeader) {
		HashMap<String, String> rs = getFileHeaderDao().getExtInfo(
				fileHeader.getGuid());
		String extPropIds;
		List<FileStoreExtInfo> extVoInfos = new ArrayList<FileStoreExtInfo>();
		try {
			extPropIds = rs.get(FileStorageDAO.EXT_PROP_COLUMN);
			String extTableNames = rs.get(FileStorageDAO.EXT_TABLENAME_COLUMN);
			if (!StringUtils.isEmpty(extPropIds)) {
				String[] splitProps = extPropIds
						.split(FileStorageDAO.SEPERATOR);
				String[] splitTableNames = extTableNames
						.split(FileStorageDAO.SEPERATOR);
				for (int i = 0; i < splitProps.length; i++) {
					String extPK = splitProps[i];
					String extTableName = splitTableNames[i];
					FileStoreExtInfo extVo = new FileStoreExtInfo();
					extVo.setExtVoPK(extPK);
					extVo.setExtVoTableName(extTableName);
					extVoInfos.add(extVo);
				}
				getFileExtPropDao().deleteExtVo(
						extVoInfos.toArray(new FileStoreExtInfo[0]));
			}
		} catch (Exception e) {
			AppDebug.error("ɾչԳ" + e);
			throw new FileStorageException("Delete extProps error!"
					+ e.getMessage());
		} finally {
			AppExecutor.postProcess();
		}
	}

	@Override
	public void deleteFileDao(FileHeader fileHeader) throws Exception {
		try {
			deleteExtVos(fileHeader);// ɾӦչ
			getFileHeaderDao().deleteFileHeader(fileHeader.getModule(),
					fileHeader.getPath());
			getFileBodyDAO().deleteFileBody(fileHeader.getGuid());
		} finally {
			AppExecutor.postProcess();
		}
	}

	@Override
	public FileHeader[] getFileHeader(String moduleId, String fileName) {
		FileHeader[] fileHeader = null;
		try {
			if (isNullModule(moduleId)) {
				moduleId = "";
			}
			fileHeader = getFileHeaderDao()
					.getHeadersByName(moduleId, fileName);
		} catch (Exception e) {
			AppDebug.error("ȡļͷϢʱ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0324")/*
																			 * @res
																			 * "ȡļͷϢ"
																			 */
					+ e.getMessage());
		}
		return fileHeader;
	}

	@Override
	public int getFileVersion(FileHeader[] fileHeaders) {
		return getFileHeaderDao().getFileVersion(fileHeaders);
	}

	/**
	 * ·ʾʽƻPK룩 Ĭpk
	 * 
	 * @return
	 */
	private boolean displayByName() {
		String display = FileStorageUtil
				.getDataConfigValue(IFileStorageConst.ATTRIBUTE_DISPLAY);
		if (!StringUtils.isEmpty(display)) {
			return Boolean.valueOf(display);
		}
		return false;
	}

	/**
	 * ģǷΪգ
	 * 
	 * @param moduleId
	 * @return
	 */
	private boolean isNullModule(String moduleId) {
		if (StringUtils.isEmpty(moduleId)
				|| ("null").equals(moduleId.toLowerCase())) {
			return true;
		}
		return false;

	}

	@Override
	public void updateFileHeader(FileHeader fileHeader, String[] attributeNames)
			throws Exception {
		try {
			getFileHeaderDao().updateFileHeader(fileHeader, attributeNames);
		} finally {
			AppExecutor.postProcess();
		}
	}

	@Override
	public void updateFileBody(FileBody fileBody) throws Exception {
		try {
			getFileBodyDAO().updateFileBody(fileBody);
		} finally {
			AppExecutor.postProcess();
		}
	}

	@Override
	public FileHeader updateMetaData(FileHeader fileHeader, FileBody fileBody,
			boolean override, String currUser, IFileStorageExt... exts)
			throws Exception {
		try {
			FileHeader updateFileHeader = null;
			if (fileHeader != null) {
				updateFileHeader = getFileHeaderDao().updateFileHeader(
						fileHeader);
			}
			if (fileBody != null) {
				getFileBodyDAO().updateFileBody(fileBody);
			}
			return updateExtVos(override, updateFileHeader, currUser, exts);
		} finally {
			AppExecutor.postProcess();
		}

	}

	@Override
	public FileHeader updateCopyMetaData(String moduleId, String newFileName,
			String currUser, FileHeader copyHeader, FileHeader fileHeader,
			FileBody fileBody, String copyStorePath, String relativePath,
			IFileStorageExt... exts) throws Exception {
		try {
			copyHeader = updateFileHeader(moduleId, newFileName, relativePath,
					copyHeader, 0, false, fileHeader.getFileSize(), currUser);
			updateFileBody(fileBody.getFiletype(), copyHeader, copyStorePath,
					fileBody.getFileSize());
			updateExtVos(false, copyHeader, currUser, exts);
		} finally {
			AppExecutor.postProcess();
		}
		return copyHeader;
	}

	@Override
	public FileHeader getFileHeaderByPath(String moduleId, String filePath)
			throws Exception {
		return getFileHeaderDao().getFileHeader(moduleId, filePath);
	}

	@Override
	public FileHeader[] getFileHeaderById(String... headIds) throws Exception {
		return getFileHeaderDao().getFileHeader(headIds);
	}

	@Override
	public FileBody getFileBody(String headId) throws Exception {
		return getFileBodyDAO().getFileBody(headId);
	}

	@Override
	public String[] getFileHeaderIds(IFileStorageExt... exts) throws Exception {
		return getFileExtPropDao().getHeaderIDs(exts);
	}

	@Override
	public FileHeader[] getAllHeaders(String... filePaths) throws Exception {
		return getFileHeaderDao().getAllHeaders(filePaths);
	}

	@Override
	public FileHeader[] getFileHeaders(String module, String... filePaths)
			throws Exception {
		return getFileHeaderDao().getFileHeaders(module, filePaths);
	}

	@Override
	public HashMap<String, String> getExtInfo(String headerId) {
		return getFileHeaderDao().getExtInfo(headerId);
	}

	@Override
	public IFileStorageExt[] getExtVo(FileStoreExtInfo... extInfos) {
		return getFileExtPropDao().getExtVo(extInfos);
	}

	private FileStorageDAO getFileHeaderDao() {
		if (fileHeaderDao == null) {
			fileHeaderDao = new FileStorageDAO();
		}
		return fileHeaderDao;
	}

	private FileBodyStorageDAO getFileBodyDAO() {
		if (fileBodyDaO == null) {
			fileBodyDaO = new FileBodyStorageDAO();
		}
		return fileBodyDaO;
	}

	private FileExtPropStorageDAO getFileExtPropDao() {
		if (fileExtPropDao == null) {
			fileExtPropDao = new FileExtPropStorageDAO();
		}
		return fileExtPropDao;
	}

}
