package uap.bs.bap.fs.storage;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import nc.bs.framework.common.InvocationInfoProxy;
import nc.bs.framework.common.NCLocator;
import nc.pub.smart.util.Chinese;
import nc.vo.pub.lang.UFDateTime;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;

import uap.bs.bap.fs.service.IFileMetaStorageService;
import uap.pub.bap.fs.FileStorageUtil;
import uap.pub.bap.fs.domain.FileBody;
import uap.pub.bap.fs.domain.FileHeader;
import uap.pub.bap.fs.domain.FileHeaderCombineModel;
import uap.pub.bap.fs.domain.FileStorageException;
import uap.pub.bap.fs.domain.IFileStorageConst;
import uap.pub.bap.fs.domain.ext.FileStoreExtInfo;
import uap.pub.bap.fs.domain.ext.IFileStorageExt;

import com.ufida.iufo.pub.tools.AppDebug;
import com.ufida.zior.perfwatch.PerfWatch;

/**
 * ļд
 * 
 * @author chenfeic @ 2013-03-14 15:48:21
 */
public class DiskFileStoreHandler implements IFileStorageHandler {

	private final String ZIP_FLAG = ".zip";// ѹ

	public DiskFileStoreHandler() {
	}

	@Override
	public FileHeader writeFile(String moduleId, String fileName,
			InputStream inputStream, String currUser,
			IFileStorageExt... fileStorageExt) throws FileStorageException {
		return writeFile(moduleId, fileName, inputStream, true, currUser,
				fileStorageExt);
	}

	@Override
	public FileHeader writeFile(String moduleId, String fileName,
			InputStream inputStream, boolean override, String currUser,
			IFileStorageExt... fileStorageExt) throws FileStorageException {
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		FileHeader uploadHeader = null;
		File file = null;// ʵʴ洢ļ
		try {
			FileHeader fileHeader = null;
			IFileMetaStorageService daoService = getDaoService();
			fileName = validateFileName(fileName);
			FileHeader[] fileHeaders = daoService.getFileHeader(moduleId,
					fileName);// ȡļԪϢ
			if (!(inputStream instanceof BufferedInputStream)) {
				inputStream = new BufferedInputStream(inputStream);
			}
			String storeDir = null;// ʵʴ洢ļĿ¼·
			String relativePath = null;// ·

			int fileVersion = daoService.getFileVersion(fileHeaders);// õļİ汾š
			boolean overWrite = overrideFile(override);
			if (!ArrayUtils.isEmpty(fileHeaders)) {
				fileHeader = fileHeaders[0];// ļôȥһļ
			}
			// ȡ·ʹ洢·Ϣ
			// ļתΪΪֹĲϵͳûаַ·Ҳ
			String enFileName = Chinese.getPinYin(fileName, false);
			storeDir = createStoreDir(moduleId, enFileName);// 洢·ļ
			relativePath = createRelativePath(fileName);

			// ǿյֻҪݿвһ¼,һҪһյļӡ⣩
			boolean emptyInputStream = FileStorageUtil
					.isEmptyInputStream(inputStream);

			if (emptyInputStream) {
				// һյļ
				file = storeFile(enFileName, null, storeDir);
				FileHeader emptyHeader = null;
				emptyHeader = daoService.updateEmptyMetaData(moduleId,
						fileName, relativePath, storeDir, fileHeader,
						fileVersion, overWrite, currUser, fileStorageExt);

				return emptyHeader;
			}

			String fileType = FileStorageUtil.getFileType(fileName);
			file = storeFile(enFileName, inputStream, storeDir);
			try {
				uploadHeader = daoService
						.updateMetaData(moduleId, fileName, fileType,
								fileHeader, fileVersion, storeDir,
								relativePath, file, overWrite, currUser,
								fileStorageExt);
			} catch (Exception e) {
				// һԴ쳣Ҫɾļ(֤Ԫʵļһ)
				FileStorageUtil.deleteFile(file);
				throw e;
			}
//			pw.appendMessage("ļдɹļPKΪ " + uploadHeader.getPath());
		} catch (Exception e) {
			AppDebug.error("дļʱ,ϢΪ" + e);
//			pw.appendMessage("дļ,ϢΪ");
//			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0327") /*
																			 * @res
																			 * "дļʱ"
																			 * *
																			 */
					+ e.getMessage());
		} finally {
//			pw.stop();
		}
		return uploadHeader;

	}

	@Override
	// TODO chenfeic һ鿼Ż
	public FileHeader update(FileHeader fileHeader, InputStream in,
			String currUser, IFileStorageExt... extVos) {
		currUser = FileStorageUtil.getCurrentUser(currUser);
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		try {
			if (fileHeader == null) {
				String msg = "fileHeader cannot be null";
				throw new FileStorageException(msg);
			}
			IFileMetaStorageService daoService = getDaoService();

			fileHeader.setLastModifytime(new UFDateTime());
			fileHeader.setLastModifier(currUser);
			if (!(in instanceof BufferedInputStream)) {
				in = new BufferedInputStream(in);
			}
			String[] attr = new String[] { FileHeader.MODIFIER,
					FileHeader.MODIFYTIME, FileHeader.NAME };
			daoService.updateFileHeader(fileHeader, attr);

			// headerзװguidֵأȥѯһ
			FileHeader updateFileHeader = daoService.getFileHeaderByPath(
					fileHeader.getModule(), fileHeader.getPath());

			if (updateFileHeader != null) {// ļ
				String headeId = updateFileHeader.getGuid();
				FileBody fileBody = daoService.getFileBody(headeId);
				if (fileBody != null) {
					String fileType = FileStorageUtil
							.getFileType(updateFileHeader.getName());
					if (StringUtils.isEmpty(fileBody.getFiletype())) {
						fileBody.setFiletype(fileType);
						daoService.updateFileBody(fileBody);
					} else if (!fileBody.getFiletype().equals(fileType)) {
						fileBody.setFiletype(fileType);
						daoService.updateFileBody(fileBody);
					}
				}
			}

			boolean emptyInputStream = FileStorageUtil.isEmptyInputStream(in);
			if (emptyInputStream) {// ļΪգܸFileheaderеļС
				if (ArrayUtils.isEmpty(extVos)) {
					return updateFileHeader;
				}
				// չԲΪգչ
				FileHeader header = daoService.updateExtVos(true,
						updateFileHeader, currUser, extVos);
				return header;
			} else {// Ϊգ˵ļи²
				FileHeader header = updateFile(updateFileHeader.getModule(),
						updateFileHeader.getPath(), currUser, in, extVos);
				return header;
			}

		} catch (Exception e) {
			AppDebug.error("ļ" + e);
			throw new FileStorageException("Update file "
					+ fileHeader.getPath() + " error:" + e.getMessage());
		} finally {
		}

	}

	@Override
	public FileHeader updateFile(String moduleId, String filePath,
			String currUser, InputStream inputStream) {
		return updateFile(moduleId, filePath, currUser, inputStream,
				new IFileStorageExt[0]);
	}

	/**
	 * ļ
	 * 
	 * @param moduleId
	 * @param filePath
	 * @param currUser
	 * @param inputStream
	 * @param extvos
	 * @return
	 */
	private FileHeader updateFile(String moduleId, String filePath,
			String currUser, InputStream inputStream, IFileStorageExt... extvos) {
		if (isNullModule(moduleId)) {
			moduleId = "";
		}
		currUser = FileStorageUtil.getCurrentUser(currUser);
		PerfWatch pw = new PerfWatch("̨ļ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼļΪ " + currUser + " ʱΪ "
				+ dateFormat.format(new Date()));
		FileHeader fileHeader = null;
		FileHeader updateHeader = null;
		try {
			IFileMetaStorageService daoService = getDaoService();
			fileHeader = daoService.getFileHeaderByPath(moduleId, filePath);// filePathĿǰ߼УǰĻΨһ~һӦȷһ¹
			if (fileHeader == null) {
				String msg = "The file does not exist! May be somebody delete this MetaData(FileHeader)";
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			String headerId = fileHeader.getGuid();
			FileBody fileBody = daoService.getFileBody(headerId);
			if (fileBody == null) {
				String msg = "Can not find the storage path of this file! May be somebody delete this MetaData(FileBody)";
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			String storagePath = fileBody.getStorePath();
			File file = null;

			String fileName = fileHeader.getName();
			String enFileName = Chinese.getPinYin(fileName, false);
			if (StringUtils.isEmpty(storagePath)) {
				// by csli:ڵһûдļд
				storagePath = createStoreDir(fileHeader.getModule(), enFileName);
				file = new File(storagePath);
				if (!file.exists()) {
					file.mkdirs();
				}
				String newStorePath = storagePath + enFileName;
				file = new File(newStorePath);
				fileBody.setStorePath(newStorePath);
			} else {
				file = new File(storagePath);
			}
			if (isZipStore(fileBody)) {// Դļʱѹ洢ģдļأѹ洢
				FileStorageUtil.zipFile(fileName, inputStream, file);// ѹļ
			} else {
				FileStorageUtil.writeFileWithoutZip(inputStream, file);
			}
			// ļϢ
			long fileLen = file.length();
			fileHeader.setFileSize(fileLen);
			fileHeader.setLastModifytime(new UFDateTime());
			fileHeader.setLastModifier(currUser);
			fileBody.setFileSize(fileLen);
			updateHeader = daoService.updateMetaData(fileHeader, fileBody,
					true, currUser, extvos);

			pw.appendMessage("ļɹ µļPKΪ " + updateHeader.getPath());
		} catch (Exception e) {
			AppDebug.error("ļ" + e);
			pw.appendMessage("ļ,ϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException("Update file " + filePath
					+ " error!" + e.getMessage());
		} finally {
			pw.stop();
		}

		return updateHeader;

	}

	@Override
	public FileHeader updateExtProps(String moduleId, String filePath,
			String currUser, IFileStorageExt... exts) {
		if (isNullModule(moduleId)) {
			moduleId = "";
		}
		currUser = FileStorageUtil.getCurrentUser(currUser);

		IFileMetaStorageService daoService = getDaoService();

		PerfWatch pw = new PerfWatch("̨չԲ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("չԣΪ " + currUser + " ʱΪ "
				+ dateFormat.format(new Date()));
		pw.appendMessage("ģΪ " + moduleId);
		pw.appendMessage("ļPKΪ " + filePath);

		FileHeader fileHeader = null;
		try {
			fileHeader = daoService.getFileHeaderByPath(moduleId, filePath);
			if (fileHeader == null) {
				String msg = "The file does not exist! May be somebody delete this MetaData(FileHeader)";
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			daoService.updateExtVos(true, fileHeader, currUser, exts);
			pw.appendMessage("չԳɹļPKΪ " + filePath);
		} catch (Exception e) {
			AppDebug.error("չԳ" + e);
			pw.appendMessage("չԳϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException("Update file" + filePath
					+ "'s extProp error!" + e.getMessage());
		} finally {
			pw.stop();
		}

		return fileHeader;
	}

	@Override
	public InputStream readFile(String moduleId, String fileId)
			throws FileStorageException {
		if (isNullModule(moduleId)) {
			moduleId = "";
		}
		InputStream in = null;
		String headerId = null;
		String storePath = null;

		PerfWatch pw = new PerfWatch("̨ļ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼļļʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģΪ" + moduleId);
		pw.appendMessage("ļPKΪ " + fileId);
		try {
			IFileMetaStorageService daoService = getDaoService();
			FileHeader fileHeader = daoService.getFileHeaderByPath(moduleId,
					fileId);
			if (fileHeader == null) {
				String msg = nc.vo.ml.NCLangRes4VoTransl.getNCLangRes()
						.getStrByID("8001006_0", "08001006-0328", null,
								new String[] { fileId })/*
														 * @ res "Դ" + fileId +
														 * "ڣ·Ƿȷ"
														 */;
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			headerId = fileHeader.getGuid();
			FileBody fileBody = daoService.getFileBody(headerId);// ȡfileBody
			if (fileBody == null) {
				String msg = "The metaData of FileBody cannot find in DB,somebody may be delete this metaData";
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			storePath = fileBody.getStorePath();
			if (StringUtils.isEmpty(storePath)) {
				return null;
			}
			// String fileName = fileHeader.getName();
			// if (isZipStore(fileBody)) {
			// in = FileStorageUtil.unZipFile(fileName, storePath);// ѹļ
			// } else {
			// ѹǰ̨ѹ
			in = FileStorageUtil.readFileWithoutZip(storePath);
			pw.appendMessage("ļɹ");
			// }
		} catch (FileNotFoundException e1) {
			AppDebug.error("ȡļ" + e1);
			pw.appendMessage("ļϢΪ");
			pw.appendMessage(e1);
			throw new FileStorageException("The file does not exist! "
					+ e1.getMessage());
			// try {
			// deleteFileDao(moduleId, fileId, headerId);
			// } catch (Exception e) {
			// AppDebug.error("ɾԪϢ", e);
			// throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
			// .getNCLangRes()
			// .getStrByID("8001006_0", "08001006-0329") /*
			// * @res
			// * "ɾԪϢ"
			// */
			// + e.getMessage());
			// }
		} catch (Exception e) {
			AppDebug.error("ȡļ" + e);
			pw.appendMessage("ļϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0330",
							null, new String[] { fileId }) /*
															 * @res "ȡļ"
															 */
					+ e.getMessage());
		} finally {
			pw.stop();
		}
		return in;
	}

	@Override
	public InputStream readFile(String moduleId, String fileId, long beginPos,
			long endPos) throws FileStorageException {
		if (isNullModule(moduleId)) {
			moduleId = "";
		}
		InputStream in = null;
		String headerId = null;
		PerfWatch pw = new PerfWatch("̨ļֶζ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼļļʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģΪ" + moduleId);
		pw.appendMessage("ļPKΪ " + fileId);
		try {
			IFileMetaStorageService daoService = getDaoService();
			FileHeader fileHeader = daoService.getFileHeaderByPath(moduleId,
					fileId);
			if (fileHeader == null) {
				String msg = nc.vo.ml.NCLangRes4VoTransl.getNCLangRes()
						.getStrByID("8001006_0", "08001006-0328", null,
								new String[] { fileId })/*
														 * @ res "Դ" + fileId +
														 * "ڣ·Ƿȷ"
														 */;
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			headerId = fileHeader.getGuid();
			FileBody fileBody = daoService.getFileBody(headerId);// ȡfileBody
			if (fileBody == null) {
				String msg = "The metaData of FileBody cannot find in DB,somebody may be delete this metaData";
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			String storePath = fileBody.getStorePath();
			if (StringUtils.isEmpty(storePath)) {
				return null;
			}
			// String fileName = fileHeader.getName();
			// if (isZipStore(fileBody)) {
			// in = FileStorageUtil.unZipFile(fileName, storePath, beginPos,
			// endPos);// ѹļ
			// } else {
			// ѹļ
			in = FileStorageUtil
					.readFileWithoutZip(storePath, beginPos, endPos);
			pw.appendMessage("ļȡɹ");
			// }
		} catch (FileNotFoundException e1) {
			AppDebug.error("ȡļ" + e1);
			pw.appendMessage("ȡļϢΪ");
			pw.appendMessage(e1);
			throw new FileStorageException("The file does not exist!"
					+ e1.getMessage());
		} catch (Exception e) {
			AppDebug.error("ȡļ" + e);
			pw.appendMessage("ȡļϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0330",
							null, new String[] { fileId }) /*
															 * @res "ȡļ"
															 */
					+ e.getMessage());
		} finally {
			pw.stop();
		}
		return in;
	}

	@Override
	public void deleteFile(String moduleId, String fileId)
			throws FileStorageException {
		if (isNullModule(moduleId)) {
			moduleId = "";
		}
		FileHeader fileHeader;
		PerfWatch pw = new PerfWatch("̨ɾļ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼɾļɾʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģΪ" + moduleId);
		pw.appendMessage("ļPKΪ:" + fileId);
		try {
			IFileMetaStorageService daoService = getDaoService();
			fileHeader = daoService.getFileHeaderByPath(moduleId, fileId);
			if (fileHeader == null) {
				pw.appendMessage("ҪɾļԪϢļͷҲ,˵ļڣ");
				return;
			}
			String headerId = fileHeader.getGuid();
			FileBody fileBody = daoService.getFileBody(headerId);// ȡfileBody
			if (fileBody == null) {
				pw.appendMessage("ҪɾļԪϢļ壩Ҳ,˵ļڣ");
				return;
			}

			String storePath = fileBody.getStorePath();
			if (!StringUtils.isEmpty(storePath)) {// ΪstorePathΪգ˵һյļûʵʵļ
				File file = new File(storePath);
				if (file.exists()) {
					FileStorageUtil.deleteFile(file);
				}
			}
			daoService.deleteFileDao(fileHeader);// ɾԪϢ

			pw.appendMessage("ɹɾļ");
		} catch (Exception e) {
			AppDebug.error("ɾļ:" + e);
			pw.appendMessage("ɾļϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0332",
							null, new String[] { fileId }) /*
															 * @res "ɾļ"
															 */
					+ e.getMessage());
		} finally {
			pw.stop();
		}
	}

	@Override
	public void renameFile(String moduleId, String fileId, String newName,
			String currUser) throws FileStorageException {
		currUser = FileStorageUtil.getCurrentUser(currUser);
		PerfWatch pw = new PerfWatch("̨ļ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ļ޸Ϊ " + currUser + " ޸ʱΪ "
				+ dateFormat.format(new Date()));
		pw.appendMessage("ģΪ" + moduleId);
		pw.appendMessage("ļPKΪ:" + fileId);

		// ᵼļѹʱ⣬Ҫ
		FileHeader fileHeader = null;
		try {
			IFileMetaStorageService daoService = getDaoService();
			if (StringUtils.isEmpty(newName)) {
				pw.appendMessage("The fileName cannot be null!");
				throw new FileStorageException("The fileName cannot be null!");
			}
			if (isNullModule(moduleId)) {
				moduleId = "";
			}
			String fileType = FileStorageUtil.getFileType(newName);
			fileHeader = daoService.getFileHeaderByPath(moduleId, fileId);
			if (fileHeader == null) {
				String msg = nc.vo.ml.NCLangRes4VoTransl.getNCLangRes()
						.getStrByID("8001006_0", "08001006-0328", null,
								new String[] { fileId })/*
														 * @ res "Դ" + fileId +
														 * "ڣ·Ƿȷ"
														 */;
				pw.appendMessage(msg);
				throw new FileStorageException(msg);
			}
			String headerId = fileHeader.getGuid();
			// ֻıԪϢ洢Ϣı
			String oldFileName = fileHeader.getName();
			if (newName.equals(oldFileName)) {
				return;
			}
			fileHeader.setName(newName);
			fileHeader.setLastModifytime(new UFDateTime());
			fileHeader.setLastModifier(currUser);
			// daoService.getFileHeaderDao().updateFileHeader(fileHeader);
			FileBody fileBody = daoService.getFileBody(headerId);// ȡfileBody
			if (fileBody != null) {
				fileBody.setFiletype(fileType);
				// daoService.getFileBodyDAO().updateFileBody(fileBody);
			}
			daoService.updateMetaData(fileHeader, fileBody, false, currUser,
					new IFileStorageExt[0]);
			pw.appendMessage("ļɹ");
		} catch (Exception e) {
			AppDebug.error("ļ" + e);
			pw.appendMessage("ļϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0336") /*
																			 * @res
																			 * *
																			 * "ļ"
																			 */
					+ e.getMessage());
		} finally {
			pw.stop();
		}
	}

	@Override
	public FileHeader[] queryHeaders(String moduleId,
			IFileStorageExt... fileStorageExt) throws FileStorageException {
		PerfWatch pw = new PerfWatch("̨ѯļͷϢһչԣ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯļͷϢѯʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģţ" + moduleId);
		pw.appendMessage("չԸ"
				+ FileStorageUtil.getExtInfos(fileStorageExt));
		FileHeader[] fileHeaders;
		try {
			if (ArrayUtils.isEmpty(fileStorageExt)) {
				return new FileHeader[0];
			}
			IFileMetaStorageService daoService = getDaoService();
			String[] headerIds = daoService.getFileHeaderIds(fileStorageExt);
			if (ArrayUtils.isEmpty(headerIds)) {
				return new FileHeader[0];
			}

			fileHeaders = daoService.getFileHeaderById(headerIds);
			StringBuilder sb = new StringBuilder();
			if (!ArrayUtils.isEmpty(fileHeaders)) {
				for (FileHeader header : fileHeaders) {
					sb.append(header.getPath()).append("_");
				}
			}
			pw.appendMessage("ѯļϢɹѯõļϢPKΪ" + sb.toString());
		} catch (Exception e) {
			AppDebug.error("ѯļͷ" + e);
			pw.appendMessage("ѯļϢϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0337") /*
																			 * @res
																			 * *
																			 * "ѯļͷϢ"
																			 */
					+ e.getMessage());
		} finally {
			pw.stop();
		}

		return fileHeaders;
	}

	@Override
	public FileHeader[] queryHeaders(String moduleId, String... filePaths)
			throws FileStorageException {
		PerfWatch pw = new PerfWatch("̨ѯļͷϢһļPK");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯļϢѯʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģţ" + moduleId);
		pw.appendMessage("ѯPK" + FileStorageUtil.getStringInfo(filePaths));
		FileHeader[] fileHeaders;
		try {
			if (ArrayUtils.isEmpty(filePaths)) {
				return new FileHeader[0];
			}
			if (isNullModule(moduleId)) {
				moduleId = "";
			}
			IFileMetaStorageService daoService = getDaoService();
			if (StringUtils.isEmpty(moduleId)) {
				fileHeaders = daoService.getAllHeaders(filePaths);
			} else {
				fileHeaders = daoService.getFileHeaders(moduleId, filePaths);
			}
			StringBuilder sb = new StringBuilder();
			if (!ArrayUtils.isEmpty(fileHeaders)) {
				for (FileHeader header : fileHeaders) {
					sb.append(header.getPath()).append("_");
				}
			}
			pw.appendMessage("ѯļϢɹļϢPKΪ" + sb.toString());
		} catch (Exception e) {
			AppDebug.error("ѯļͷ" + e);
			pw.appendMessage("ѯļϢϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0337") /*
																			 * @res
																			 * *
																			 * "ѯļͷϢ"
																			 */
					+ e.getMessage());
		} finally {
			pw.stop();
		}
		return fileHeaders;
	}

	@Override
	public FileHeaderCombineModel[] queryHeaderCombModel(String moduleId,
			String... filePaths) throws FileStorageException {
		PerfWatch pw = new PerfWatch("̨ѯԪģ(һļPK)");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯչԣѯʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģţ" + moduleId);
		pw.appendMessage("ѯPK" + FileStorageUtil.getStringInfo(filePaths));
		FileHeaderCombineModel[] fileHeaderCombModel;
		try {
			FileHeader[] fileHeaders = queryHeaders(moduleId, filePaths);
			fileHeaderCombModel = getFileHeaderCombModel(fileHeaders);
		} finally {
			pw.stop();
		}
		return fileHeaderCombModel;
	}

	@Override
	public FileHeaderCombineModel[] queryHeaderCombModel(String moduleId,
			IFileStorageExt... fileStorageExt) throws FileStorageException {
		PerfWatch pw = new PerfWatch("̨ѯԪģ(һչ)");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯչԣѯʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģţ" + moduleId);
		pw.appendMessage("ѯչԸ"
				+ FileStorageUtil.getExtInfos(fileStorageExt));
		try {
			FileHeader[] fileHeaders = queryHeaders(moduleId, fileStorageExt);
			return getFileHeaderCombModel(fileHeaders);
		} finally {
			pw.stop();
		}
	}

	@Override
	public IFileStorageExt[] queryExtProps(String moduleId, String... filePaths)
			throws FileStorageException {
		PerfWatch pw = new PerfWatch("̨ѯչ(һPK)");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯչԣѯʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ģţ" + moduleId);
		pw.appendMessage("ѯPK" + FileStorageUtil.getStringInfo(filePaths));
		FileHeader[] headers;
		try {
			if (ArrayUtils.isEmpty(filePaths)) {
				return new IFileStorageExt[0];
			}
			if (isNullModule(moduleId)) {
				moduleId = "";
			}
			IFileMetaStorageService daoService = getDaoService();
			if (StringUtils.isEmpty(moduleId)) {
				headers = daoService.getAllHeaders(filePaths);
			} else {
				headers = daoService.getFileHeaders(moduleId, filePaths);
			}
		} catch (Exception e) {
			AppDebug.error("ѯļͷ" + e);
			pw.appendMessage("ѯļͷϢԭΪ");
			pw.appendMessage(e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0337") /*
																			 * @res
																			 * *
																			 * "ѯļͷϢ"
																			 */
					+ e.getMessage());
		} finally {
			pw.stop();
		}
		return queryExtProps(headers);
	}

	@Override
	public IFileStorageExt[] queryExtProps(FileHeader... fileHeader)
			throws FileStorageException {
		PerfWatch pw = new PerfWatch("̨ѯļչԣһԪݣ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯչԣѯʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ѯԪݸ"
				+ FileStorageUtil.getHeaderInfo(fileHeader));
		try {
			if (ArrayUtils.isEmpty(fileHeader)) {
				return new IFileStorageExt[0];
			}
			IFileMetaStorageService daoService = getDaoService();
			List<FileStoreExtInfo> extList = new ArrayList<FileStoreExtInfo>();
			for (FileHeader header : fileHeader) {
				String headerId = header.getGuid();
				HashMap<String, String> rs = daoService.getExtInfo(headerId);// 
				String extPropIds = rs.get(FileStorageDAO.EXT_PROP_COLUMN);
				String extTableNames = rs
						.get(FileStorageDAO.EXT_TABLENAME_COLUMN);
				String clzNames = rs.get(FileStorageDAO.EXT_TYPE_COLUMN);
				if (StringUtils.isEmpty(extTableNames)
						|| StringUtils.isEmpty(extPropIds)
						|| StringUtils.isEmpty(clzNames)) {
					return new IFileStorageExt[0];
				}

				String[] splitExtPks = extPropIds
						.split(FileStorageDAO.SEPERATOR);
				String[] splitTableNames = extTableNames
						.split(FileStorageDAO.SEPERATOR);
				String[] splitClzName = clzNames
						.split(FileStorageDAO.SEPERATOR);
				for (int i = 0; i < splitExtPks.length; i++) {
					String extPk = splitExtPks[i];
					String extTableName = splitTableNames[i];
					String clzName = splitClzName[i];
					FileStoreExtInfo extInfo = new FileStoreExtInfo();
					extInfo.setExtVoPK(extPk);
					extInfo.setExtVoTableName(extTableName);
					extInfo.setExtVoType(clzName);
					extList.add(extInfo);
				}
			}
			FileStoreExtInfo[] extInfos = extList.toArray(new FileStoreExtInfo[0]);
			IFileStorageExt[] extVo = daoService.getExtVo(extInfos);
			pw.appendMessage("ѯɹѯõչԸΪ" + extVo.length);
			return extVo;
		} finally {
			pw.stop();
		}
	}

	@Override
	public FileHeader copy(String moduleId, String filePath,
			String newFileName, String currUser, IFileStorageExt... exts)
			throws FileStorageException {
		FileHeader copyHeader = null;
		if (isNullModule(moduleId)) {
			moduleId = "";// ȫģ
		}
		currUser = FileStorageUtil.getCurrentUser(currUser);
		PerfWatch pw = new PerfWatch("̨ļ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼļΪ " + currUser + " ʱΪ "
				+ dateFormat.format(new Date()));
		try {
			IFileMetaStorageService daoService = getDaoService();
			FileHeader fileHeader = daoService.getFileHeaderByPath(moduleId,
					filePath);
			if (fileHeader == null) {
				pw.appendMessage("The file need to delete does not exist!");
				throw new FileStorageException(
						"The file need to delete does not exist!");
			}
			if (StringUtils.isEmpty(newFileName)) {
				newFileName = fileHeader.getName();
			}
			FileBody fileBody = daoService.getFileBody(fileHeader.getGuid());
			String sourceStorePath = fileBody.getStorePath();
			String copyStorePath = null;
			String enNewFileName = Chinese.getPinYin(newFileName, false);
			copyStorePath = createStoreDir(moduleId, enNewFileName)
					+ enNewFileName;// ǵüļ
			// ļ
			FileStorageUtil.copyFile(sourceStorePath, copyStorePath);

			String relativePath = createRelativePath(newFileName);
			copyHeader = daoService.updateCopyMetaData(moduleId, newFileName,
					currUser, copyHeader, fileHeader, fileBody, copyStorePath,
					relativePath, exts);
			pw.appendMessage("ļƳɹ,ļģΪ" + copyHeader.getModule()+ "ļPKΪ" + copyHeader.getPath());
		} catch (Exception e) {
			AppDebug.error("ļ" + e);
			pw.appendMessage("ļϢΪ");
			pw.appendMessage(e);
			throw new FileStorageException("copy file error!" + e.getMessage());
		} finally {
			pw.stop();
		}

		return copyHeader;
	}

	@Override
	public FileHeader copy(String module, String filePath, String newModule,
			String newFileName, String currUser, IFileStorageExt... exts)
			throws FileStorageException {
		if (isNullModule(module)) {
			module = "";// Ϊnull,ȫģ
		}
		currUser = FileStorageUtil.getCurrentUser(currUser);
		PerfWatch pw = new PerfWatch("̨ģ鸴Ʋ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼļΪ " + currUser + " ʱΪ "
				+ dateFormat.format(new Date()));
		FileHeader copyHeader = null;
		try {
			IFileMetaStorageService daoService = getDaoService();
			FileHeader fileHeader = daoService.getFileHeaderByPath(module,
					filePath);
			if (fileHeader == null) {
				pw.appendMessage("The file need to delete does not exist!");
				throw new FileStorageException(
						"The file need to delete does not exist!");
			}
			if (StringUtils.isEmpty(newFileName)) {
				newFileName = fileHeader.getName();
			}
			String enNewFileName = Chinese.getPinYin(newFileName, false);
			FileBody fileBody = daoService.getFileBody(fileHeader.getGuid());
			String sourceStorePath = fileBody.getStorePath();
			String copyStorePath = null;
			copyStorePath = createStoreDir(newModule, enNewFileName)
					+ enNewFileName;// ǵüļ׺
			// ļ
			FileStorageUtil.copyFile(sourceStorePath, copyStorePath);
			String relativePath = createRelativePath(newFileName);
			copyHeader = daoService.updateCopyMetaData(newModule, newFileName,
					currUser, copyHeader, fileHeader, fileBody, copyStorePath,
					relativePath, exts);
			pw.appendMessage("ģ鸴ļɹ,ļģΪ" + copyHeader.getModule() + "ļPKΪ" + copyHeader.getPath());
		} catch (Exception e) {
			AppDebug.error("ģ鸴Ƴ" + e);
			pw.appendMessage("ģ鸴ƳԭΪ");
			pw.appendMessage(e);
			throw new FileStorageException("copy file error!" + e.getMessage());
		} finally {
			pw.stop();
		}
		return copyHeader;
	}

	/**
	 * ļд
	 * 
	 * @param fileName
	 * @param inputStream
	 * @param parentStorePath
	 * @return
	 */
	private File storeFile(String fileName, InputStream inputStream,
			String parentStorePath) {
		File file = null;
		File storeDir = new File(parentStorePath);// ʵʵĴ洢·
		// Ŀ¼ڣҪһ
		if (!storeDir.exists()) {
			storeDir.mkdirs();
		}
		if (needCompress()) {// һҪӦó֮ǰûѹдѹˣʱжļ
			String zipFileName = fileName.split("\\.")[0] + ".zip";// ѹļ,
			file = new File(parentStorePath, zipFileName);// ѹ洢ļ
			FileStorageUtil.zipFile(fileName, inputStream, file);// ѹļ
		} else {
			file = new File(parentStorePath, fileName);
			FileStorageUtil.writeFileWithoutZip(inputStream, file);// дļҪѹ
		}
		return file;
	}

	/**
	 * ģļʵĴ洢Ŀ¼
	 * ·򣺸·/ģ/û/a/b/ʱ/abcd.txt(2Ŀ¼Ŀ¼ļǰ
	 * 磬abc.txtɵ2Ŀ¼a/b)
	 * 
	 * @param moduleId
	 *            ģ
	 * @param fileName
	 * @return
	 */
	private String createStoreDir(String moduleId, String fileName) {
		if (isNullModule(moduleId)) {
			moduleId = IFileStorageConst.DEFAULT_MODULE;
		}
		if (StringUtils.isEmpty(fileName)) {
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0338") /*
																			 * @res
																			 * *
																			 * "Ƿļ"
																			 */);
		}
		String restPath = FileStorageUtil
				.getDataConfigValue(IFileStorageConst.ATTRIBUTE_FILEPATH);// Ŀ¼
		if (StringUtils.isEmpty(restPath)) {
			restPath = FileStorageUtil
					.getDataConfigValue(IFileStorageConst.DEFAULT_FILE_REPO);// ûøĿ¼ʹĬ·
		}
		if (!restPath.endsWith(File.separator)) {
			restPath = restPath + File.separator;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(restPath).append(moduleId).append(File.separator)
				.append(InvocationInfoProxy.getInstance().getUserId());

		int index = fileName.lastIndexOf(".");
		if (index == -1) {// ûк׺
			sb.append("fs").append(File.separator).append("noSuffix")
					.append(File.separator);
		} else {
			String prefixName = fileName.substring(0, index);
			if (prefixName.length() < 2) {
				sb.append(File.separator).append(fileName.toLowerCase())
						.append(File.separator);
			} else {
				char firstPath = prefixName.toLowerCase().charAt(0);
				char secondPath = prefixName.toLowerCase().charAt(1);
				if (firstPath == '.') {
					firstPath = 'a';
				}
				if (secondPath == '.') {
					secondPath = 'b';
				}
				sb.append(File.separator).append(firstPath)
						.append(File.separator).append(secondPath)
						.append(File.separator);
			}

		}
		sb.append(FileStorageUtil.getCurrTime()).append(File.separator);// ½һĿ¼
		return sb.toString();
	}

	/**
	 * ·ɹ
	 * 
	 * @param fileName
	 * @return ·Ϊʱļ
	 */
	private String createRelativePath(String fileName) {
		StringBuilder sb = new StringBuilder();
		sb.append(FileStorageUtil.getCurrTime()).append(fileName);
		return sb.toString();
	}

	private FileHeaderCombineModel[] getFileHeaderCombModel(
			FileHeader[] fileHeaders) {
		PerfWatch pw = new PerfWatch("̨ѯļԪģͣһԪϢ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯļԪģͣѯʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ԪݸΪ" + FileStorageUtil.getHeaderInfo(fileHeaders));
		List<FileHeaderCombineModel> headerModelList = new ArrayList<FileHeaderCombineModel>();
		try {
			if (ArrayUtils.isEmpty(fileHeaders)) {
				return new FileHeaderCombineModel[0];
			}

			for (FileHeader header : fileHeaders) {
				IFileStorageExt[] extVos = queryExtProps(header);
				FileHeaderCombineModel fileHeaderModel = new FileHeaderCombineModel(
						header);
				fileHeaderModel.setExtVos(extVos);
				headerModelList.add(fileHeaderModel);
			}
			StringBuilder sb = new StringBuilder();
			if (headerModelList.size() != 0) {
				for (FileHeaderCombineModel combineModel : headerModelList) {
					sb.append(combineModel.getFileHeader().getPath()).append(
							"_");
				}
			}
			pw.appendMessage("ѯļԪģͳɹѯļPKΪ " + sb.toString());

		} catch (Exception e) {
			AppDebug.error("ѯļͷģͳ" + e);
			pw.appendMessage("ѯļͷģͳԭΪ");
			pw.appendMessage(e);
			throw new FileStorageException("query Header error!"
					+ e.getMessage());
		} finally {
			pw.stop();
		}
		return headerModelList.toArray(new FileHeaderCombineModel[0]);
	}

	/**
	 * ļ---ȥӦط
	 * 
	 * @param fileName
	 * @return
	 */
	private String validateFileName(String fileName) {
		if (StringUtils.isEmpty(fileName)) {
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0338") /*
																			 * @res
																			 * *
																			 * "Ƿļ"
																			 */);
		}
		String name = null;
		int index = fileName.lastIndexOf(File.separator);
		if (index == -1) {
			int index2 = fileName.lastIndexOf("/");// windowsҲִַָ
			if (index2 == -1) {
				name = fileName;
			} else {
				name = fileName.substring(index2 + 1, fileName.length());
			}
		} else {
			name = fileName.substring(index + 1, fileName.length());
		}
		// ļǷϷ

		if (!name
				.matches("[^\\s\\\\/:\\*\\?\\\"<>\\|](\\x20|[^\\s\\\\/:\\*\\?\\\"<>\\|])*[^\\s\\\\/:\\*\\?\\\"<>\\|\\.]$")) {
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0338") /*
																			 * @res
																			 * *
																			 * "Ƿļ"
																			 */);
		}
		return name;
	}

	/**
	 * ǷҪѹ
	 * 
	 * @return
	 */
	private boolean needCompress() {
		String needComPress = FileStorageUtil
				.getDataConfigValue(IFileStorageConst.ATTRIBUTE_COMPRESS);
		if (!StringUtils.isEmpty(needComPress)) {
			return Boolean.valueOf(needComPress);
		}
		return false;
	}

	/**
	 * Ƿдļ(ģΪλ)
	 */
	private Boolean overrideModelFile() {
		String overWrite = FileStorageUtil
				.getDataConfigValue(IFileStorageConst.ATTRIBUTE_OVERRIDE);
		if (!StringUtils.isEmpty(overWrite)) {
			return Boolean.valueOf(overWrite);
		}
		return true;
	}

	/**
	 * Ƿдļ(ļ)
	 * 
	 * @param override
	 * @return
	 */
	private boolean overrideFile(boolean override) {
		boolean overWrite;
		if (!override) {// жļǷҪ󸲸
			overWrite = override;
		} else {// ļΪǣ鿴ģǷҪ󸲸
			overWrite = overrideModelFile();
		}
		return overWrite;
	}

	/**
	 * жļǷѹ洢
	 * 
	 * @param fileBody
	 * @return
	 */
	private boolean isZipStore(FileBody fileBody) {
		if (fileBody == null) {
			return false;
		}
		String storePath = fileBody.getStorePath();
		if (StringUtils.isEmpty(storePath)) {
			return false;
		}
		return storePath.toLowerCase().endsWith(ZIP_FLAG);
	}

	/**
	 * ģǷΪգ
	 * 
	 * @param moduleId
	 * @return
	 */
	private boolean isNullModule(String moduleId) {
		if (StringUtils.isEmpty(moduleId)
				|| ("null").equals(moduleId.toLowerCase())) {
			return true;
		}
		return false;

	}

	@Override
	public boolean isExist(String moduleId, String filePath) {
		PerfWatch pw = new PerfWatch("̨жļǷ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼļǷڣʱΪ" + dateFormat.format(new Date()));
		pw.appendMessage("ѯģΪ" + moduleId);
		pw.appendMessage("ѯļPKΪ" + filePath);
		try {
			if (StringUtils.isEmpty(filePath)) {
				pw.appendMessage("filePathΪ");
				return false;
			}
			IFileMetaStorageService daoService = getDaoService();
			if (isNullModule(moduleId)) {
				moduleId = "";
			}
			FileHeader fileHeader = daoService.getFileHeaderByPath(moduleId,
					filePath);
			if (fileHeader == null) {
				pw.appendMessage("fileHeaderΪ");
				return false;
			} else {
				if (fileHeader.getFileSize() == 0) {
					pw.appendMessage("ļСΪ0");
					return false;// ļfalse
				}
				FileBody fileBody = daoService
						.getFileBody(fileHeader.getGuid());
				if (fileBody == null) {
					pw.appendMessage("fileBodyΪ");
					return false;
				} else {
					String storePath = fileBody.getStorePath();
					if (StringUtils.isEmpty(storePath)) {
						pw.appendMessage("洢·Ϊ");
						return false;
					} else {
						File file = new File(storePath);
						if (!file.exists()) {
							pw.appendMessage("ļ");
							return false;
						} else {
							if (file.length() == 0) {
								return false;
							} else {
								return true;
							}
						}

					}
				}
			}
		} catch (Exception e) {
			AppDebug.error("ѯļǷڳ" + e);
			pw.appendMessage("ļǷڳԭΪ");
			pw.appendMessage(e);
			throw new FileStorageException("Test file exists error"
					+ e.getMessage());
		} finally {
			pw.stop();
		}

	}

	@Override
	public IFileMetaStorageService getDaoService() {
		IFileMetaStorageService daoService = NCLocator.getInstance().lookup(
				IFileMetaStorageService.class);
		return daoService;
	}

}
