package uap.bs.bap.fs.storage;

import java.lang.reflect.Field;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import nc.bs.logging.Logger;
import nc.jdbc.framework.ConnectionFactory;
import nc.jdbc.framework.JdbcSession;
import nc.jdbc.framework.PersistenceManager;
import nc.jdbc.framework.SQLParameter;
import nc.jdbc.framework.crossdb.CrossDBConnection;
import nc.jdbc.framework.generator.SequenceGenerator;
import nc.jdbc.framework.mapping.IMappingMeta;
import nc.jdbc.framework.processor.BeanMappingListProcessor;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.jdbc.framework.util.DBConsts;
import nc.jdbc.framework.util.DBUtil;
import nc.jdbc.framework.util.SQLHelper;
import nc.pub.smart.util.SmartUtilities;
import nc.util.iufo.pub.OIDMaker;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;

import uap.pub.aert.datasource.DataSourceUtil;
import uap.pub.bap.fs.domain.FileStorageConfigFactory;
import uap.pub.bap.fs.domain.FileStorageException;
import uap.pub.bap.fs.domain.ext.FileStorageExtPackage;
import uap.pub.bap.fs.domain.ext.FileStorageMappingMeta;
import uap.pub.bap.fs.domain.ext.FileStoreExtInfo;
import uap.pub.bap.fs.domain.ext.IFileStorageExt;

import com.ufida.dataset.db.DbType;
import com.ufida.iufo.pub.tools.AppDebug;
import com.ufida.zior.perfwatch.PerfWatch;

/**
 * չDao
 * 
 * @author chenfeic
 * 
 */
public class FileExtPropStorageDAO {

	public static final String EXT_GUID = "guid";

	public static final String EXT_TABLENAME = "tableName";

	public static final String EXT_TYPE = "type";

	public static final String EXT_PK = "fld_id";

	/** չԱheaderId **/
	public static final String HEADER_COLUMN = "fld_headerId";

	public boolean isTableExisted(IMappingMeta meta) throws Exception {

		String tableName = meta.getTableName();
		String sql = "select count(1) from  sysobjects where name ='"
				+ tableName + "' and type='U'";

		JdbcSession jdbc = null;
		int iCount = -1;
		PersistenceManager persist = null;
		try {
			String dsName = FileStorageConfigFactory.getConfig().getDsName();
			persist = PersistenceManager.getInstance(dsName);
			jdbc = persist.getJdbcSession();

			// ݿͣЧŻ
			int iDBType = jdbc.getDbType();
			if (iDBType == DBConsts.SQLSERVER) {
				sql = "select count(1) from  sysobjects where name ='"
						+ tableName + "' and type='U'";
				jdbc.setSQLTranslator(false);
			} else if (iDBType == DBConsts.ORACLE) {
				sql = "select count(1) from user_tables where table_name ='"
						+ tableName.toUpperCase() + "'";
				jdbc.setSQLTranslator(false);
			} else if (iDBType == DBConsts.DB2) {
				sql = "select count(1) from syscat.tables where tabschema = current schema and tabname = '"
						+ tableName.toUpperCase() + "'";
				jdbc.setSQLTranslator(false);
			} else if (iDBType == DBConsts.POSTGRESQL) {
				sql = "select count(1) from pg_class where relname = '"
						+ tableName.toLowerCase() + "'";// shit~~Сд
				jdbc.setSQLTranslator(false);
			}
			// ѯ
			Integer count = (Integer) jdbc.executeQuery(sql,
					new ColumnProcessor(1));
			iCount = count.intValue();
		} catch (Exception e) {
			Logger.error("жչԶӦıǷʱ");
			Logger.error("[View existed]: ͨϵͳʱ", e);
			return false;
		} finally {
			if (persist != null)
				persist.release();
		}
		return (iCount != 0);
	}

	/**
	 * չԶӦı
	 * 
	 * @param meta
	 */
	public void createTable(IMappingMeta meta) {
		String[] columns = meta.getColumns();
		String[] columnTypes = ((FileStorageMappingMeta) meta).getColumnTypes();
		if (ArrayUtils.isEmpty(columns) || ArrayUtils.isEmpty(columnTypes)
				|| columns.length != columnTypes.length) {
			String errorMsg = nc.vo.ml.NCLangRes4VoTransl.getNCLangRes()
					.getStrByID("8001006_0", "08001006-0354") /*
															 * @res *
															 * "еĸжӦ͸һ"
															 */;
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0342") /*
																			 * @res
																			 * *
																			 * "չԶӦıʧ"
																			 */
					+ errorMsg);
		}
		String tableName = meta.getTableName().toUpperCase();
		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		String pk = meta.getPrimaryKey();

		String attrSql = "";
		for (int i = 0; i < columns.length; i++) {// pk
			attrSql += columns[i] + " " + columnTypes[i] + ", ";
		}
		// headerIdֶ
		attrSql += HEADER_COLUMN + " " + "VARCHAR(20)" + ", ";
		String sql = "CREATE TABLE " + tableName + "(" + pk
				+ " CHAR(20) NOT NULL, " + attrSql + " " + " PRIMARY KEY "
				+ "(" + pk + "))";

		JdbcSession jdbc = null;
		PersistenceManager persist = null;
		try {
			persist = PersistenceManager.getInstance(dsName);
			jdbc = persist.getJdbcSession();
			jdbc.executeUpdate(sql);
		} catch (Exception e) {
			AppDebug.error("չԶӦıʧ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0342") /*
																			 * @res
																			 * *
																			 * "չԶӦıʧ"
																			 */
					+ e.getMessage());
		} finally {
			if (jdbc != null)
				jdbc.closeAll();
			if (persist != null)
				persist.release();
		}

	}
	
    public void createTable(FileStorageExtPackage... extMetas) throws Exception {
        if (ArrayUtils.isEmpty(extMetas)) {
            return;
        }

        JdbcSession jdbc = null;
        PersistenceManager persist = null;
        try {
            String dsName = FileStorageConfigFactory.getConfig().getDsName();
            persist = PersistenceManager.getInstance(dsName);
            jdbc = persist.getJdbcSession();
            for (FileStorageExtPackage extMeta : extMetas) {
                String sql = genCreateSql(extMeta, dsName);
                jdbc.addBatch(sql);
            }
            jdbc.executeBatch();
        } catch (Exception e) {
            AppDebug.error("create ext domain table failed.");
            // throw new FileStorageDBException(e.getMessage());
        } finally {
            DataSourceUtil.postProcess(true);
            closeSessionAndPersit(persist, jdbc);
        }

    }
    
    private String genCreateSql(FileStorageExtPackage extMeta, String dsName) {
        String[] columns = extMeta.getColumnsName();
        String[] columnTypes = extMeta.getColumnsType();
        if (ArrayUtils.isEmpty(columns) || ArrayUtils.isEmpty(columnTypes) || columns.length != columnTypes.length) {
            String errorMsg = nc.vo.ml.NCLangRes4VoTransl.getNCLangRes().getStrByID("0302001_0", "00302001-0032") /*
                                                                                                                   * @
                                                                                                                   * res
                                                                                                                   * *
                                                                                                                   * "еĸжӦ͸һ"
                                                                                                                   */;
            throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl.getNCLangRes().getStrByID("0302001_0",
                    "00302001-0020") /*
                                      * @res
                                      * *
                                      * "չԶӦıʧ"
                                      */
                    + errorMsg);
        }
        String tableName = extMeta.getTableName().toUpperCase();

        String pk = extMeta.getPrimaryKey();

        StringBuilder attrSql = new StringBuilder();
        for (int i = 0; i < columns.length; i++) {// pk
            attrSql.append(columns[i]).append(" ").append(columnTypes[i]).append(", ");
        }
        // headerIdֶ
        attrSql.append(HEADER_COLUMN).append(" ").append("VARCHAR(20)").append(", ");
        // tsֶ
        attrSql.append(addTSSql(dsName)).append(",");
        // drֶ
        attrSql.append(addDRSql(dsName)).append(",");

        return "CREATE TABLE " + tableName + "(" + pk + " CHAR(20) NOT NULL, " + attrSql.toString() + " "
                + " PRIMARY KEY " + "(" + pk + "))";
    }

    private String addTSSql(String dsName) {
        String sql = "ts  CHAR(19)";
        DbType dbType = SmartUtilities.getDbType(dsName);
        if (dbType == DbType.SQLSERVER) {
            sql = "ts NCHAR(19) COLLATE Chinese_PRC_CI_AS DEFAULT CONVERT(nchar(19),GETDATE(),20)";
        } else if (dbType == DbType.ORACLE) {
            sql = "ts CHAR(19) DEFAULT to_char(sysdate,'yyyy-mm-dd hh24:mi:ss')";
        } else if (dbType == DbType.DB2) {
            sql = "ts CHAR(19) null";
        } else if (dbType == DbType.POSTGRESQL) {
            sql = "ts  CHAR(19) DEFAULT to_char(current_timestamp,'yyyy-mm-dd hh24:mi:ss')";
        }
        return sql;
    }

    private String addDRSql(String dsName) {
        String sql = "dr numeric(10,0) DEFAULT 0";
        DbType dbType = SmartUtilities.getDbType(dsName);
        if (dbType == DbType.SQLSERVER || dbType == DbType.DB2) {
            sql = "dr SMALLINT DEFAULT 0";
        }
        if (dbType == DbType.DB2) {
            sql = "dr SMALLINT null DEFAULT 0";
        }
        return sql;
    }
	/**
	 * ѯչ
	 * 
	 * @param extPk
	 *            PKֵ
	 * @param tableName
	 *            
	 * @param clzName
	 *            չԵ
	 * @return
	 */
	public IFileStorageExt getExtVo(String extPk, String tableName,
			String clzName) {
		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		PersistenceManager persist = null;
		JdbcSession jdbcSession = null;
		try {
			persist = PersistenceManager.getInstance(dsName);
			jdbcSession = persist.getJdbcSession();
			jdbcSession.setSQLTranslator(false);// ִз롣
			String sql = "Select * from " + tableName + " Where " + EXT_PK
					+ " = ?";
			SQLParameter params = new SQLParameter();
			params.addParam(extPk);
			Class<?> clz = Class.forName(clzName);
			IFileStorageExt extVO = (IFileStorageExt) clz.newInstance();
			Collection retrieve = (Collection) jdbcSession.executeQuery(sql,
					params, new BeanMappingListProcessor(clz,
							new FileStorageMappingMeta(extVO)));
			Iterator iterator = retrieve.iterator();
			IFileStorageExt next = null;
			while (iterator.hasNext()) {// ֻһ
				next = (IFileStorageExt) iterator.next();
			}
			return next;
		} catch (Exception e) {
			AppDebug.error("ѯչԳ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0350") /*
																			 * @res
																			 * "ѯչԳ"
																			 */
					+ e.getMessage());
		} finally {
			if (jdbcSession != null)
				jdbcSession.closeAll();
			if (persist != null)
				persist.release();
		}
	}

	/**
	 * ѯչ
	 * 
	 * @param extInfos
	 *            һļչϢ
	 * @return
	 */
	public IFileStorageExt[] getExtVo(FileStoreExtInfo... extInfos) {
		if (ArrayUtils.isEmpty(extInfos)) {
			return new IFileStorageExt[0];
		}
		List<IFileStorageExt> extList = new ArrayList<IFileStorageExt>();
		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		PersistenceManager persist = null;
		JdbcSession jdbcSession = null;
		try {
			persist = PersistenceManager.getInstance(dsName);
			jdbcSession = persist.getJdbcSession();
			jdbcSession.setSQLTranslator(false);// ִз롣
			for (FileStoreExtInfo extInfo : extInfos) {
				String sql = SQLHelper.getSelectSQL(extInfo.getExtVoTableName(),
						null, new String[] { EXT_PK });
				SQLParameter params = new SQLParameter();
				params.addParam(extInfo.getExtVoPK());
				Class<?> clz = Class.forName(extInfo.getExtVoType());
				IFileStorageExt extVO = (IFileStorageExt) clz.newInstance();
				Collection retrieve = (Collection) jdbcSession.executeQuery(
						sql, params, new BeanMappingListProcessor(clz,
								new FileStorageMappingMeta(extVO)));
				Iterator iterator = retrieve.iterator();
				IFileStorageExt next = null;
				while (iterator.hasNext()) {// ֻһ
					next = (IFileStorageExt) iterator.next();
					extList.add(next);
				}
			}
			return extList.toArray(new IFileStorageExt[0]);
		} catch (Exception e) {
			AppDebug.error("ѯչԳ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0350") /*
																			 * @res
																			 * "ѯչԳ"
																			 */
					+ e.getMessage());
		} finally {
			if (jdbcSession != null)
				jdbcSession.closeAll();
			if (persist != null)
				persist.release();
		}
	}

	/**
	 * չϢ
	 * 
	 * @param headerId
	 *            չԶӦheadrpkֵ
	 * @param extVos
	 * @param meta
	 * @return صǲݵpk,Ϣ
	 */
	public FileStoreExtInfo[] insertObject(String headerId,
			IFileStorageExt... extVos) {
		if (ArrayUtils.isEmpty(extVos)) {
			return new FileStoreExtInfo[0];
		}
		List<FileStoreExtInfo> extInfos = new ArrayList<FileStoreExtInfo>();

		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		PersistenceManager persist = null;
		JdbcSession jdbcSession = null;

		try {
			persist = PersistenceManager.getInstance(dsName);
			jdbcSession = persist.getJdbcSession();
			jdbcSession.setSQLTranslator(false);// ִз롣
			jdbcSession.setAddTimeStamp(false);// Ҫts
			for (IFileStorageExt extVo : extVos) {
				IMappingMeta meta = new FileStorageMappingMeta(extVo);
				List<String> validateColumns = new ArrayList<String>();

				String pk = OIDMaker.getGE(
						FileStorageConfigFactory.getConfig().getDsName())
						.generate();// ֶPK
				String tableName = meta.getTableName().toUpperCase();
				String[] columns = meta.getColumns();// 
				// ѯsqlPK
				validateColumns.add(meta.getPrimaryKey());
				SQLParameter parameter = new SQLParameter();
				// мpk
				parameter.addParam(pk);
				String[] attributes = meta.getAttributes();
				for (int i = 0; i < attributes.length; i++) {
					// õԵֵ
					Field field = extVo.getClass().getDeclaredField(
							attributes[i]);
					field.setAccessible(true);
					Object value = field.get(extVo);
					if (value != null) {// ֵΪգ򲻲
						validateColumns.add(columns[i]);
						parameter.addParam(value);
					}
				}
				validateColumns.add(HEADER_COLUMN);// headerId
				parameter.addParam(headerId);
				String insertSQL = SQLHelper.getInsertSQL(tableName,
						validateColumns.toArray(new String[0]));
				jdbcSession.addBatch(insertSQL, parameter);
				FileStoreExtInfo extInfo = new FileStoreExtInfo();
				extInfo.setExtVoPK(pk);
				extInfo.setExtVoTableName(tableName);
				extInfo.setExtVoType(extVo.getClass().getName());
				extInfos.add(extInfo);
			}
			jdbcSession.executeBatch();
			return extInfos.toArray(new FileStoreExtInfo[0]);
		} catch (Exception e) {
			AppDebug.error("չϢ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0351") /*
																			 * @res
																			 * *
																			 * "չϢ"
																			 */
					+ e.getMessage());
		} finally {
			if (jdbcSession != null)
				jdbcSession.closeAll();
			if (persist != null)
				persist.release();
		}
	}
	
	public FileStoreExtInfo[] insertObjectNew(String headerId, FileStorageExtPackage... extVos) throws Exception {
        if (ArrayUtils.isEmpty(extVos)) {
            return new FileStoreExtInfo[0];
        }
        List<FileStoreExtInfo> extInfos = new ArrayList<FileStoreExtInfo>();
        String dsName = FileStorageConfigFactory.getConfig().getDsName();
        PersistenceManager persist = null;
        JdbcSession jdbcSession = null;
        try {
            persist = PersistenceManager.getInstance(dsName);
            jdbcSession = persist.getJdbcSession();
            jdbcSession.setSQLTranslator(true);
             jdbcSession.setAddTimeStamp(false);// Ҫts
            for (FileStorageExtPackage extVo : extVos) {
                List<String> validateColumns = new ArrayList<String>();
                String corpPk = SQLHelper.getCorpPk();
                String[] pks = new SequenceGenerator(dsName).generate(corpPk, 1);
                String tableName = extVo.getTableName().toUpperCase();
                String[] columns = extVo.getColumnsName();// 
                Object[] values = extVo.getColumnsValue();// ֵ
                // ѯsqlPK
                validateColumns.add(extVo.getPrimaryKey());
                SQLParameter parameter = new SQLParameter();
                // мpk
                parameter.addParam(pks[0]);
                for (int i = 0; i < columns.length; i++) {
                    if (values[i] != null) {// ֵΪգ򲻲
                        validateColumns.add(columns[i]);
                        parameter.addParam(values[i]);
                    }
                }
                validateColumns.add(HEADER_COLUMN);// headerId
                parameter.addParam(headerId);
                String insertSQL = SQLHelper.getInsertSQL(tableName, validateColumns.toArray(new String[0]));
                jdbcSession.addBatch(insertSQL, parameter);
                FileStoreExtInfo extInfo = new FileStoreExtInfo();
                extInfo.setExtVoPK(pks[0]);
                extInfo.setExtVoTableName(tableName);
                extInfo.setExtVoType(extVo.getExtClassNmae());
                extInfos.add(extInfo);
            }
            jdbcSession.executeBatch();
            return extInfos.toArray(new FileStoreExtInfo[0]);
        } catch (Exception e1) {
            AppDebug.error("batch insert ext doamin data error.");
            throw e1;
        } finally {
            closeSessionAndPersit(persist, jdbcSession);
        }
    }

	  private void closeSessionAndPersit(PersistenceManager persist, JdbcSession jdbcSession) {
	        if (jdbcSession != null) {
	            jdbcSession.closeAll();
	        }
	        if (persist != null) {
	            persist.release();
	        }
	    }
	  
	/**
	 * һչԣȡӦͷϢid
	 * 
	 * @param fileExts
	 * @return
	 */
	public String[] getHeaderIDs(IFileStorageExt... fileExts) {
		if (ArrayUtils.isEmpty(fileExts)) {
			return new String[0];
		}
		List<String> headList = new ArrayList<String>();
		for (IFileStorageExt fileExt : fileExts) {
			String[] headerIds = getHeaderID(fileExt);
			if (!ArrayUtils.isEmpty(headerIds)) {
				for (String headerId : headerIds) {
					headList.add(headerId);
				}
			}
		}
		return headList.toArray(new String[0]);
	}

	/**
	 * չԻȡӦͷϢID ȻֻһչԣǸֵʵԲѯheader¼
	 * 
	 * @param fileExt
	 * @return
	 */
	private String[] getHeaderID(IFileStorageExt fileExt) {
		PerfWatch pw = new PerfWatch("̨ѯļͷϢbyչԣFileExtPropStorageDAO.getHeaderID(IFileStorageExt)");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ʼѯļϢѯʱΪ" + dateFormat.format(new Date()));
		if (fileExt == null) {
			return new String[0];
		}
		IMappingMeta meta = new FileStorageMappingMeta(fileExt);

		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		ResultSet rs = null;
		Connection conn = null;
		List<String> headerList = new ArrayList<String>();
		try {
			if (!isTableExisted(meta)) {// չԶӦıڣֱӷnull
				return new String[0];
			}
			conn = getConnection(dsName);
			pw.appendMessage("ԴΪ" + dsName);
			// õSQL
			String sql = "SELECT " + HEADER_COLUMN + " FROM "
					+ meta.getTableName() + " Where ";
			List<String> validateColumns = new ArrayList<String>();
			SQLParameter params = new SQLParameter();
			String[] attributes = meta.getAttributes();
			String[] columns = meta.getColumns();
			for (int i = 0; i < attributes.length; i++) {
				// õԵֵ
				Field field = fileExt.getClass()
						.getDeclaredField(attributes[i]);
				field.setAccessible(true);
				Object value = field.get(fileExt);
				if (value != null) {// ֵΪգ򲻲
					if (!isZeroValue(value)) {
						validateColumns.add(columns[i]);
						params.addParam(value);
					}
				}
			}
			int size = validateColumns.size();
			if (size == 0) {
				return new String[0];
			}
			for (int i = 0; i < size; i++) {
				if (i != size - 1) {
					sql += validateColumns.get(i) + "=? And ";
				} else {
					sql += validateColumns.get(i) + "=?";
				}
			}
			pw.appendMessage("ѯsqlΪ" + sql);
			List parameters = params.getParameters();
			StringBuilder sb = new StringBuilder();
			if (parameters != null) {
				for (int i = 0; i < parameters.size(); i++) {
					sb.append(parameters.get(i)).append(",");
				}
			}
			pw.appendMessage("ѯsqlΪ" + sb.toString());
			PreparedStatement prepStatement = conn.prepareStatement(sql);
			prepStatement.clearParameters();
			if (params != null) {
				DBUtil.setStatementParameter(prepStatement, params);
			}
			prepStatement.setMaxRows(100000);
			prepStatement.setFetchSize(40);
			rs = prepStatement.executeQuery();
			StringBuilder sb1 = new StringBuilder();
			while (rs.next()) {
				String headerId = rs.getString(HEADER_COLUMN);
				if (!StringUtils.isEmpty(headerId)) {
					headerList.add(headerId);
					sb1.append(headerId).append(",");
				}
			}
			pw.appendMessage("ѯɹ,ѯļͷΪ" + sb1.toString());
			return headerList.toArray(new String[0]);
		} catch (Exception e) {
			AppDebug.error("ȡչӦͷϢ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0352") /*
																			 * @res
																			 * "ȡչӦͷϢ"
																			 */
					+ e.getMessage());
		} finally {
			DBUtil.closeRs(rs);
			DBUtil.closeConnection(conn);
			pw.stop();
		}
	}

	/**
	 * жϵǰֵǷΪͣΪ0
	 * 
	 * @param value
	 * @return
	 */
	private boolean isZeroValue(Object value) {
		if (value instanceof Integer && ((Integer) value).intValue() == 0)
			return true;
		if (value instanceof Long && ((Long) value).intValue() == 0)
			return true;
		if (value instanceof Short && ((Short) value).intValue() == 0)
			return true;
		return false;

	}

	/**
	 * ɾչ עʹʹô˷ʱչԵPKһһӦᵼɾʧ
	 * 
	 * @param extPks
	 *            չԵPKֵ
	 * @param tableName
	 *            չԶӦı
	 */
	public void deleteExtVo(FileStoreExtInfo[] extVoInfos) {
		if (ArrayUtils.isEmpty(extVoInfos)) {
			return;
		}
		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		PersistenceManager persist = null;
		JdbcSession jdbcSession = null;
		try {
			persist = PersistenceManager.getInstance(dsName);
			jdbcSession = persist.getJdbcSession();
			jdbcSession.setSQLTranslator(false);// ִз롣
			for (FileStoreExtInfo extVoInfo : extVoInfos) {
				String sql = "Delete  from " + extVoInfo.getExtVoTableName()
						+ " Where " + EXT_PK + " = ?";
				SQLParameter params = new SQLParameter();
				params.addParam(extVoInfo.getExtVoPK());
				jdbcSession.addBatch(sql, params);
			}

			jdbcSession.executeBatch();
		} catch (Exception e) {
			AppDebug.error("ɾչʱ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0353") /*
																			 * @res
																			 * "ɾչʱ"
																			 */
					+ e.getMessage());
		} finally {
			if (jdbcSession != null)
				jdbcSession.closeAll();
			if (persist != null)
				persist.release();
		}
	}

	/**
	 * ȡݿ
	 * 
	 * @param dsName
	 * @return
	 * @throws SQLException
	 */
	private Connection getConnection(String dsName) throws SQLException {
		CrossDBConnection con = (CrossDBConnection) ConnectionFactory
				.getConnection(dsName);
		con.enableSQLTranslator(false);
		con.setAddTimeStamp(false);
		return con;
	}
}
