package uap.bs.bap.fs.storage;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;

import nc.bs.iufo.dao.DAO;
import nc.bs.iufo.dao.DAOAction;
import nc.jdbc.framework.ConnectionFactory;
import nc.jdbc.framework.JdbcSession;
import nc.jdbc.framework.PersistenceManager;
import nc.jdbc.framework.SQLParameter;
import nc.jdbc.framework.crossdb.CrossDBConnection;
import nc.jdbc.framework.util.DBUtil;
import nc.pub.ba.util.StringUtil;
import nc.pub.iufo.annotation.DBTableName;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;

import uap.pub.bap.fs.domain.FileHeader;
import uap.pub.bap.fs.domain.FileStorageConfigFactory;
import uap.pub.bap.fs.domain.FileStorageException;

import com.ufida.iufo.pub.tools.AppDebug;
import com.ufida.zior.perfwatch.PerfWatch;

/**
 * ļԪϢ洢DAO
 * 
 * @author csli
 * 
 */
public class FileStorageDAO {
	/** ֶηָ */
	public final static String SEPERATOR = "@@@";

	/** չԱʶ */
	public final static String EXT_PROP_COLUMN = "ext_props";
	/** չԶӦı */
	public final static String EXT_TABLENAME_COLUMN = "ext_tablename";
	/** չ */
	public final static String EXT_TYPE_COLUMN = "ext_type";

	/**
	 * ģ룬ļ߼ѯļͷ
	 * 
	 * @param module
	 *            ģ
	 * @param filePath
	 *            ļ߼
	 * @return
	 * @throws Exception
	 */
	public FileHeader getFileHeader(String module, String filePath)
			throws Exception {
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		String clauseStr = getClauseStr(module, filePath);
		try {
			Object[] headers = new DAOAction()
					.loadByClause(FileHeader.class, FileStorageConfigFactory
							.getConfig().getDsName(), clauseStr);
			if (!ArrayUtils.isEmpty(headers)) {
				return (FileHeader) headers[0];
			} else {
				return null;
			}
		} finally {
		}
	}

	public FileHeader[] getFileHeaders(String module, String... filePaths)
			throws Exception {
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
//		String clauseStr = "";
		StringBuilder sb = new StringBuilder();
		
		if (!StringUtils.isEmpty(module)) {
			sb.append("module='").append(module).append("' and ");
//			clauseStr += "module='" + module + "' and ";
		}
		String inClause = StringUtil.getInClause(filePaths, "path");
		sb.append(inClause);
		
//		int length = filePaths.length;
//		for (int i = 0; i < length; i++) {
//			clauseStr += "path='" + filePaths[i] + "'";
//			if (i != length - 1) {
//				clauseStr += "OR ";
//			}
//		}
		try {
			Object[] headers = new DAOAction()
					.loadByClause(FileHeader.class, FileStorageConfigFactory
							.getConfig().getDsName(), sb.toString());

			return (FileHeader[]) headers;
		} finally {
		}
	}

	/**
	 * õģfileHeader
	 * 
	 * @param module
	 * @param filePaths
	 * @return
	 * @throws Exception
	 */
	public FileHeader[] getAllHeaders(String... filePaths) throws Exception {

		PerfWatch pw = new PerfWatch("HeaderDaoģȡһļͷ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ϴʱΪ" + dateFormat.format(new Date()));

		if (ArrayUtils.isEmpty(filePaths)) {
			return new FileHeader[0];
		}
//		String clauseStr = "";
		String clauseStr = StringUtil.getInClause(filePaths, "path");
		
//		int length = filePaths.length;
//		for (int i = 0; i < length; i++) {
//			clauseStr += "path='" + filePaths[i] + "'";
//			if (i != length - 1) {
//				clauseStr += "OR ";
//			}
//		}
		pw.appendMessage("ѯsqlΪ" + clauseStr);
		try {
			Object[] headers = (FileHeader[]) new DAOAction().loadByClause(
					FileHeader.class, FileStorageConfigFactory.getConfig()
							.getDsName(), clauseStr);
			return (FileHeader[]) headers;
		} finally {
			pw.stop();
		}
	}

	/**
	 * ļͷidѯļͷ
	 * 
	 * @param headerId
	 * @return
	 * @throws Exception
	 */
	public FileHeader[] getFileHeader(String... headerIds) throws Exception {
		PerfWatch pw = new PerfWatch("HeaderDaoģȡһļͷ()");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ϴʱΪ" + dateFormat.format(new Date()));

		if (ArrayUtils.isEmpty(headerIds)) {
			return new FileHeader[0];
		}
//		String clauseStr = "";
//		int length = headerIds.length;
//		for (int i = 0; i < length; i++) {
//			clauseStr += "guid='" + headerIds[i] + "'";
//			if (i != length - 1) {
//				clauseStr += "OR ";
//			}
//		}
		String clauseStr = StringUtil.getInClause(headerIds, "guid"); 
		
		pw.appendMessage("ѯsqlΪ" + clauseStr);
		try {
			Object[] headers = new DAOAction()
					.loadByClause(FileHeader.class, FileStorageConfigFactory
							.getConfig().getDsName(), clauseStr);
			return (FileHeader[]) headers;
		} finally {
			pw.stop();
		}
	}

	/**
	 * õļͬļļ汾
	 * 
	 * @param module
	 * @param name
	 * @return
	 */
	public int getFileVersion(String module, String name) {
		FileHeader[] headers = getHeadersByName(module, name);
		return getFileVersion(headers);

	}

	/**
	 * õһļͷеĵļ汾
	 * 
	 * @param headers
	 * @return
	 */
	public int getFileVersion(FileHeader... headers) {
		if (ArrayUtils.isEmpty(headers)) {
			return 0;
		} else {
			int tmpVersion = headers[0].getFileVersion();
			for (FileHeader header : headers) {
				int fileVersion = header.getFileVersion();
				if (fileVersion > tmpVersion) {
					tmpVersion = fileVersion;
				}
			}
			return tmpVersion;
		}
	}

	/**
	 * ļģţļѯļͷϢ
	 * 
	 * @param module
	 * @param name
	 * @return
	 */
	public FileHeader[] getHeadersByName(String module, String name) {
//		PerfWatch pw = new PerfWatch("HeaderDaoļѯļͷϢ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
//		pw.appendMessage("ϴʱΪ" + dateFormat.format(new Date()));

		String clauseStrByName = getClauseStrByName(module, name);
//		pw.appendMessage("ѯsqlΪ" + clauseStrByName);
		try {
			Object[] headers = new DAOAction().loadByClause(FileHeader.class,
					FileStorageConfigFactory.getConfig().getDsName(),
					clauseStrByName);
			return (FileHeader[]) headers;
		} finally {
//			pw.stop();
		}
	}

	/**
	 * ݿ½һļͷϢ
	 * 
	 * @param fileHeader
	 * @throws Exception
	 */
	public FileHeader createFileHeader(FileHeader fileHeader) throws Exception {
		Object[] objs = new DAOAction().add(FileHeader.class,
				FileStorageConfigFactory.getConfig().getDsName(), fileHeader);
		if (ArrayUtils.isEmpty(objs)) {
			return null;
		} else {
			return (FileHeader) objs[0];
		}
	}

	/**
	 * ɾļͷ
	 * 
	 * @param module
	 * @param path
	 * @throws Exception
	 */
	public void deleteFileHeader(String module, String path) throws Exception {
		PerfWatch pw = new PerfWatch("HeaderDaoɾļͷϢ");
		DateFormat dateFormat = DateFormat.getDateTimeInstance();
		pw.appendMessage("ϴʱΪ" + dateFormat.format(new Date()));
		String clauseStr = getClauseStr(module, path);

		pw.appendMessage("ɾsqlΪ" + clauseStr);
		try {
			new DAOAction()
					.removeByClause(FileHeader.class, FileStorageConfigFactory
							.getConfig().getDsName(), clauseStr);
		} finally {
			pw.stop();
		}
	}

	/**
	 * ȡչԵϢ
	 * 
	 * @param extPk
	 * @param tableName
	 * @return
	 */
	public HashMap<String, String> getExtInfo(String headerId) {
		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		ResultSet rs = null;
		HashMap<String, String> extMap = new HashMap<String, String>();
		Connection conn = null;
		try {
			String sql = "Select " + EXT_PROP_COLUMN + ","
					+ EXT_TABLENAME_COLUMN + "," + EXT_TYPE_COLUMN + " from "
					+ getTableName() + " Where guid = ?";
			SQLParameter params = new SQLParameter();
			params.addParam(headerId);
			conn = getConnection(dsName);
			PreparedStatement prepStatement = conn.prepareStatement(sql);
			prepStatement.clearParameters();
			if (params != null) {
				DBUtil.setStatementParameter(prepStatement, params);
			}
			prepStatement.setMaxRows(100000);
			prepStatement.setFetchSize(40);
			rs = prepStatement.executeQuery();
			while (rs.next()) {
				String ext_props = rs.getString(EXT_PROP_COLUMN);
				String extTableName = rs.getString(EXT_TABLENAME_COLUMN);
				String extType = rs.getString(EXT_TYPE_COLUMN);
				extMap.put(EXT_PROP_COLUMN, ext_props);
				extMap.put(EXT_TABLENAME_COLUMN, extTableName);
				extMap.put(EXT_TYPE_COLUMN, extType);
			}
			return extMap;
		} catch (Exception e) {
			AppDebug.error("ѯչԳ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0350") /*
																			 * @res
																			 * "ѯչԳ"
																			 */
					+ e.getMessage());
		} finally {
			DBUtil.closeRs(rs);
			DBUtil.closeConnection(conn);
		}
	}

	/**
	 * һչϢ
	 * 
	 * @param headerId
	 * @param extProps
	 * @param extTables
	 * @param extTypes
	 */
	public void insertExtInfo(String headerId, String extProps,
			String extTables, String extTypes) {
		if (StringUtils.isEmpty(headerId) || StringUtils.isEmpty(extTypes)
				|| StringUtils.isEmpty(extTables)
				|| StringUtils.isEmpty(extProps)) {
			return;
		}
		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		PersistenceManager persist = null;
		JdbcSession jdbcSession = null;
		try {
			persist = PersistenceManager.getInstance(dsName);
			jdbcSession = persist.getJdbcSession();
			jdbcSession.setSQLTranslator(false);// ִз롣
			String sql = "update " + getTableName() + " set " + EXT_PROP_COLUMN
					+ " = ?, " + EXT_TABLENAME_COLUMN + " = ?, "
					+ EXT_TYPE_COLUMN + " = ? where guid = ?";
			SQLParameter parameter = new SQLParameter();
			parameter.addParam(extProps);
			parameter.addParam(extTables);
			parameter.addParam(extTypes);
			parameter.addParam(headerId);
			jdbcSession.executeUpdate(sql, parameter);
		} catch (Exception e) {
			AppDebug.error("չԳ" + e);
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0351") /*
																			 * @res
																			 * *
																			 * "չԳ"
																			 */
					+ e.getMessage());
		} finally {
			if (jdbcSession != null)
				jdbcSession.closeAll();
			if (persist != null)
				persist.release();
		}
	}

	/**
	 * ļͷϢ
	 * 
	 * @param fileHeader
	 * @throws Exception
	 */
	public FileHeader updateFileHeader(FileHeader fileHeader) throws Exception {
		if (fileHeader == null) {
			return null;
		} else {
			Object[] update = new DAOAction().update(FileHeader.class,
					FileStorageConfigFactory.getConfig().getDsName(),
					fileHeader);
			if (ArrayUtils.isEmpty(update)) {
				return null;
			} else {
				return (FileHeader) update[0];
			}

		}
	}

	/**
	 * ָϢ
	 * 
	 * @param fileHeader
	 * @param attributeNames
	 *            ҪµϢ
	 * @throws Exception
	 */
	public void updateFileHeader(FileHeader fileHeader, String[] attributeNames)
			throws Exception {
		if (fileHeader == null) {
			return;
		}
		String dsName = FileStorageConfigFactory.getConfig().getDsName();
		DAO dao = null;
		if (dsName == null) {
			dao = new DAO(FileHeader.class);
		} else {
			dao = new DAO(FileHeader.class, dsName);
		}
		String module = fileHeader.getModule();
		String path = fileHeader.getPath();

		String whereClause = getClauseStr(module, path);

		dao.update(new FileHeader[] { fileHeader }, attributeNames, whereClause);
	}

	private String getClauseStr(String module, String path) {
		if (StringUtils.isEmpty(module)
				|| ("null").equals(module.toLowerCase())) {
			return "path='" + path + "'";
		}
		return "module = '" + module + "' and path='" + path + "'";
	}

	private String getClauseStrByName(String module, String name) {
		if (StringUtils.isEmpty(module)
				|| ("null").equals(module.toLowerCase())) {
			return "name='" + name + "'";
		}
		return "module = '" + module + "' and name='" + name + "'";
	}

	/**
	 * õFileHeaderӦݿ
	 * 
	 * @return
	 */
	private String getTableName() {
		Class<? extends FileHeader> clz = FileHeader.class;
		DBTableName name = clz.getAnnotation(DBTableName.class);
		if (name == null) {
			throw new FileStorageException(
					"The table of fileHeader is not supported!");
		}
		return name.value();
	}

	/**
	 * ȡݿ
	 * 
	 * @param dsName
	 * @return
	 * @throws SQLException
	 */
	private Connection getConnection(String dsName) throws SQLException {
		CrossDBConnection con = (CrossDBConnection) ConnectionFactory
				.getConnection(dsName);
		con.enableSQLTranslator(false);
		con.setAddTimeStamp(false);
		return con;
	}

}
