package uap.pub.bap.fs;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.zip.Deflater;
import java.util.zip.ZipOutputStream;

import nc.bs.framework.common.InvocationInfoProxy;
import nc.pub.iufo.zip.CNZipOutputStream;
import nc.pub.iufo.zip.ZipFileUtil;

import org.apache.commons.configuration.DataConfiguration;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;

import uap.pub.bap.fs.domain.FileHeader;
import uap.pub.bap.fs.domain.FileStorageConfigFactory;
import uap.pub.bap.fs.domain.FileStorageException;
import uap.pub.bap.fs.domain.IFileStorageConst;
import uap.pub.bap.fs.domain.ext.BaFileStoreExt;
import uap.pub.bap.fs.domain.ext.FileStorageExtName;
import uap.pub.bap.fs.domain.ext.FileStorageExtPackage;
import uap.pub.bap.fs.domain.ext.FileStorageExtProp;
import uap.pub.bap.fs.domain.ext.FileStorageMappingMeta;
import uap.pub.bap.fs.domain.ext.IFileStorageExt;

import com.ufida.iufo.pub.tools.AppDebug;

public class FileStorageUtil implements IFileStorageConst {

	private final static String BAK_FLAG = ".aebak";// ʱļ׺

	/**
	 * ֵ
	 * 
	 * @param extProps
	 * @return
	 */
	public static String joinExtProps(String... extProps) {
		if (ArrayUtils.isEmpty(extProps)) {
			return null;
		} else {
			StringBuilder sbd = new StringBuilder();
			for (int i = 0; i < extProps.length; i++) {
				if (i == extProps.length - 1)
					sbd.append(extProps[i]);
				else
					sbd.append(extProps[i]).append(EXT_CONNECTOR);
			}
			return sbd.toString();
		}

	}

	/**
	 * ȡֵ
	 * 
	 * @param propStr
	 * @return
	 */
	public static String[] splitPropStr(String propStr) {
		if (StringUtils.isEmpty(propStr)) {
			return new String[0];
		} else {
			return propStr.split(EXT_CONNECTOR);
		}

	}

	/**
	 * ѹļ Ŀǰ߼򵥣ֵ֧ǵļѹ
	 * 
	 * @param fileName
	 *            ѹڲļ
	 * @param in
	 * @param zipFile
	 *            ѹļ
	 * @throws IOException
	 */
	public static void zipFile(String fileName, InputStream in, File zipFile) {
		OutputStream out = null;
		try {
			CNZipOutputStream zos = new CNZipOutputStream(new FileOutputStream(
					zipFile, false), "GBK");
			zos.setMethod(ZipOutputStream.DEFLATED);// ѹ
			zos.setLevel(Deflater.BEST_COMPRESSION);// ǿѹǻʱ϶
			out = new BufferedOutputStream(zos);
			zos.putNextEntry(fileName);
			if (in == null) {// һյļ
				zipFile.createNewFile();
				return;
			}

			int len = -1;
			byte[] buf = new byte[2048];// 2k
			while ((len = in.read(buf)) != -1) {
				out.write(buf, 0, len);
			}
			out.flush();
		} catch (IOException e) {
			throw new FileStorageException(
					nc.vo.ml.NCLangRes4VoTransl.getNCLangRes().getStrByID(
							"8001006_0", "08001006-0346") /* @res "ļѹ" */, e);
		} finally {
			closeOutputStream(out);
			closeInputStream(in);
		}
	}

	/**
	 * ѹдļ
	 * 
	 * @param in
	 * @param file
	 * @throws IOException
	 */
	public static void writeFileWithoutZip(InputStream in, File file) {
		BufferedOutputStream out = null;
		try {
			out = new BufferedOutputStream(new FileOutputStream(file, false));
			if (in == null) {// һյļ
				file.createNewFile();
				return;
			}
			int len = -1;
			byte[] buf = new byte[2048];// 2k
			while ((len = in.read(buf)) != -1) {
				out.write(buf, 0, len);
			}
			out.flush();
		} catch (IOException e) {
			throw new FileStorageException(
					nc.vo.ml.NCLangRes4VoTransl.getNCLangRes().getStrByID(
							"8001006_0", "08001006-0347") /* @res "дļ" */, e);
		} finally {
			closeOutputStream(out);
			closeInputStream(in);
		}

	}

	/**
	 * ѹļ
	 * 
	 * @param entryName
	 * @param zipPath
	 * @return
	 * @throws IOException
	 * @throws
	 * @throws IOException
	 */
	public static InputStream unZipFile(String entryName, String zipPath)
			throws IOException {
		InputStream inputStream = ZipFileUtil.getInputStream(new File(zipPath),
				entryName);
		return new BufferedInputStream(inputStream);
	}

	/**
	 * ѹļ
	 * 
	 * @param entryName
	 * @param zipPath
	 * @return
	 * @throws IOException
	 */
	public static InputStream unZipFile(String entryName, String zipPath,
			long beginPos, long endPos) throws IOException {
		InputStream inputStream = ZipFileUtil.getInputStream(new File(zipPath),
				entryName);
		InputStream skipInputStream = skipInputStream(beginPos, endPos,
				inputStream);
		return skipInputStream;

	}

	/**
	 * ļ
	 * 
	 * @param sourceFileName
	 *            ҪƵļƣ·
	 * @param targetFileName
	 *            ƺļ·
	 */
	public static void copyFile(String sourceFileName, String targetFileName) {
		if (StringUtils.isEmpty(targetFileName)) {
			throw new FileStorageException("The name of file cannot be null!");
		}
		File targetFile = new File(targetFileName);
		File parentFile = targetFile.getParentFile();
		if (parentFile != null && !parentFile.exists()) {
			parentFile.mkdirs();
		}
		if (StringUtils.isEmpty(sourceFileName)) {// Դļڣһյļ
			try {
				targetFile.createNewFile();
				return;
			} catch (IOException e) {
				AppDebug.error("ļƳ:" + e);
				throw new FileStorageException("Copy file error:"
						+ e.getMessage());
			}
		}

		File sourceFile = new File(sourceFileName);
		if (!sourceFile.exists()) {
			throw new FileStorageException(nc.vo.ml.NCLangRes4VoTransl
					.getNCLangRes().getStrByID("8001006_0", "08001006-0328",
							null, new String[] { sourceFileName }) /*
																	 * @ res
																	 * "Դ" +
																	 * sourceFileName
																	 * +
																	 * "ڣ·Ƿȷ"
																	 */);
		}

		copyFile(sourceFile, targetFile);
	}

	/**
	 * ļ
	 * 
	 * @param sourceFile
	 *            ҪƵļ
	 * @param targetFile
	 *            ƺļ
	 */
	public static void copyFile(File sourceFile, File targetFile) {
		if (!sourceFile.exists()) {
			throw new FileStorageException("The file must be exist!");
		}
		// ½ļл
		FileInputStream input = null;
		BufferedInputStream inBuff = null;
		FileOutputStream output = null;
		BufferedOutputStream outBuff = null;
		try {
			input = new FileInputStream(sourceFile);
			inBuff = new BufferedInputStream(input);

			// ½ļл
			output = new FileOutputStream(targetFile);
			outBuff = new BufferedOutputStream(output);

			// 
			byte[] b = new byte[1024 * 5];
			int len;
			while ((len = inBuff.read(b)) != -1) {
				outBuff.write(b, 0, len);
			}
			// ˢ´˻
			outBuff.flush();
		} catch (Exception e) {
			AppDebug.error(e);
			throw new FileStorageException("Copy file error!", e);

		} finally {
			closeInputStream(inBuff);
			closeOutputStream(outBuff);
			closeOutputStream(output);
			closeInputStream(input);
		}
	}

	/**
	 * ȡѹļ
	 * 
	 * @param filePath
	 * @return
	 * @throws IOException
	 */
	public static InputStream readFileWithoutZip(String filePath)
			throws IOException {
		BufferedInputStream bin = null;
		bin = new BufferedInputStream(new FileInputStream(new File(filePath)));
		return bin;
	}

	/**
	 * 
	 * @param filePath
	 * @param beginPos
	 * @param endPos
	 * @return
	 * @throws IOException
	 */
	public static InputStream readFileWithoutZip(String filePath,
			long beginPos, long endPos) throws IOException {
		FileInputStream inputStream = new FileInputStream(new File(filePath));
		InputStream skipInputStream = skipInputStream(beginPos, endPos,
				inputStream);
		return skipInputStream;

	}

	/**
	 * 
	 * @param beginPos
	 * @param endPos
	 * @param inputStream
	 * @return
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	public static InputStream skipInputStream(long beginPos, long endPos,
			InputStream inputStream) {
		BufferedOutputStream tmpOut = null;
		InputStream skipInputStream = null;
		try {
			if (beginPos > endPos) {
				throw new IOException("endPos must be bigger than beginPos!");
			}
			inputStream.skip(beginPos);
			int len = -1;
			byte[] buf = new byte[2048];
			File tmpFile = File.createTempFile("tmp", null);
			tmpFile.deleteOnExit();

			tmpOut = new BufferedOutputStream(new FileOutputStream(tmpFile));

			long length = endPos - beginPos + 1;
			long times = length / buf.length;
			long remainder = length % buf.length;
			for (long i = 0; i < times; i++) {
				len = inputStream.read(buf, 0, buf.length);
				if (len != -1) {
					tmpOut.write(buf, 0, len);
					tmpOut.flush();
				} else {
					break;
				}
			}
			if (remainder > 0) {
				len = inputStream.read(buf, 0, (int) remainder);
				if (len != -1) {
					tmpOut.write(buf, 0, len);
					tmpOut.flush();
				}
			}
			skipInputStream = new BufferedInputStream(new FileInputStream(
					tmpFile));
		} catch (Exception e) {
			throw new FileStorageException(
					nc.vo.ml.NCLangRes4VoTransl.getNCLangRes().getStrByID(
							"8001006_0", "08001006-0348") /* @res "ļ" */, e);
		} finally {
			closeInputStream(inputStream);
			closeOutputStream(tmpOut);
		}
		return skipInputStream;
	}

	/**
	 * ɾĿ¼ļ
	 * 
	 * @param file
	 * @return
	 */
	public static boolean deleteFile(File file) {
		if (!file.exists()) {
			return false;
		}
		if (file.isFile()) {
			return file.delete();
		} else {
			boolean success = true;
			File[] childs = file.listFiles();
			int count = childs == null ? 0 : childs.length;
			for (int i = 0; i < count; i++) {
				success &= deleteFile(childs[i]);
			}
			success &= file.delete();
			return success;
		}
	}

	/**
	 * ȡļ
	 * 
	 * @param fileName
	 * @return
	 */
	public static String getFileType(String fileName) {
		if (StringUtils.isEmpty(fileName)) {
			return null;
		}
		int index = fileName.lastIndexOf(".");
		if (index == -1) {
			return null;
		}
		String suffixName = fileName.substring(index + 1, fileName.length())
				.toUpperCase();

		return suffixName;

	}

	/**
	 * ˷дļж жϵǰǷ㹻Ŀռдļ
	 * 
	 * @param file
	 *            ļ
	 * @param diskName
	 */
	public static void checkDiskUsable(File file, String diskName) {
		long fileSize = file.length();
		long diskUsableSize = getDiskUsableSize(diskName);
		if (fileSize >= diskUsableSize) {
			// TODO Ҫ
			throw new FileStorageException(
					"There is not enough disk space,please change anthor disk");
		}

	}

	/**
	 * 
	 * @param diskName
	 *             C: C:\\XXX(ļ)
	 * @return
	 */
	private static long getDiskUsableSize(String diskName) {
		File file = new File(diskName);
		return file.getUsableSpace();
	}

	public static void closeInputStream(InputStream in) {
		try {
			if (in != null) {
				in.close();
			}
		} catch (IOException e) {
			AppDebug.error(e);
		}

	}

	public static void closeOutputStream(OutputStream out) {
		try {
			if (out != null) {
				out.close();
			}
		} catch (IOException e) {
			AppDebug.error(e);
		}
	}

	/**
	 * жǷΪ
	 * 
	 * @param in
	 * @return
	 * @throws IOException
	 */
	public static boolean isEmptyInputStream(InputStream in) throws IOException {
		in.mark(1);
		int count = in.read();
		in.reset();
		return (count == -1);
	}

	/**
	 * ȡǰû
	 * 
	 * @param currUser
	 * @return
	 */
	public static String getCurrentUser(String currUser) {
		if (StringUtils.isEmpty(currUser)
				|| "null".equals(currUser.toLowerCase())) {
			currUser = InvocationInfoProxy.getInstance().getUserId();
		}
		return currUser;
	}

	/**
	 * ȡչ
	 * 
	 * @param ext
	 * @return
	 */
	public static String getExtTable(IFileStorageExt ext) {
		if(ext instanceof FileStorageExtPackage){
			return ((FileStorageExtPackage) ext).getTableName();
		}
		Class<? extends IFileStorageExt> clz = ext.getClass();
		FileStorageExtName name = clz.getAnnotation(FileStorageExtName.class);
		if (name == null) {
			throw new FileStorageException(ext.getClass().getName()
					+ " is not supported!");
		}
		return FileStorageMappingMeta.PERFIX_TABLE + name.value();
	}

	/**
	 * ȡչvoпɲݿ
	 * 
	 * @param ext
	 * @return
	 */
	public static String[] getExtAttr(IFileStorageExt ext) {
		Class<? extends IFileStorageExt> clz = ext.getClass();
		List<String> attributes = new ArrayList<String>();
		Field[] flds = clz.getDeclaredFields();
		for (Field fld : flds) {
			FileStorageExtProp attr = (FileStorageExtProp) fld
					.getAnnotation(FileStorageExtProp.class);
			if (attr != null) {
				attributes.add(attr.value());
			}
		}
		return attributes.toArray(new String[0]);
	}

	/**
	 * õǰʱ
	 * 
	 * @return
	 */
	private static Lock l = new ReentrantLock();

	public static String getCurrTime() {
		l.lock();
		long currentTimeMillis;
		try {
			currentTimeMillis = System.currentTimeMillis();
			try {
				TimeUnit.MILLISECONDS.sleep(100);// ͣ100
			} catch (InterruptedException e) {
				AppDebug.error(e);
			}
		} finally {
			l.unlock();
		}
		return String.valueOf(currentTimeMillis);
	}

	/**
	 * ȡԵ תΪӦݿ
	 * 
	 * @param ext
	 * @return
	 */
	public static String[] getExtAttrType(IFileStorageExt ext) {
		Class<? extends IFileStorageExt> clz = ext.getClass();
		List<String> types = new ArrayList<String>();
		Field[] flds = clz.getDeclaredFields();
		for (Field fld : flds) {
			FileStorageExtProp attr = (FileStorageExtProp) fld
					.getAnnotation(FileStorageExtProp.class);
			if (attr != null) {
				String type = fld.getGenericType().toString();
				if (type.equals("class java.lang.String")) {
					types.add("VARCHAR(100)");
				} else if (type.equals("class java.lang.Character")
						|| type.equals("char")) {
					types.add("VARCHAR(10)");
				} else if (type.equals("class java.lang.Integer")
						|| type.equals("int")
						|| type.equals("class java.lang.Short")
						|| type.equals("short")
						|| type.equals("class java.lang.Long")
						|| type.equals("long")) {
					types.add("numeric(30,0)");
				}
			}
		}
		return types.toArray(new String[0]);
	}

	/**
	 * ܼҪ
	 * 
	 * @param exts
	 * @return
	 */
	public static String getExtInfos(IFileStorageExt[] exts) {
		int len;
		if (ArrayUtils.isEmpty(exts)) {
			len = 0;
		} else {
			len = exts.length;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(len);
		sb.append(".");
		for (int i = 0; i < len; i++) {
			if (exts[i] instanceof BaFileStoreExt) {
				sb.append("Ext").append(i).append("'s billItem=");
				String pk_billitem = ((BaFileStoreExt) exts[i])
						.getPk_billitem();
				sb.append(pk_billitem).append("_");
			}

		}

		return sb.toString();
	}

	/**
	 * ܼҪ
	 * 
	 * @param exts
	 * @return
	 */
	public static String getHeaderInfo(FileHeader[] headers) {
		int len;
		if (ArrayUtils.isEmpty(headers)) {
			len = 0;
		} else {
			len = headers.length;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(len);
		sb.append(".");
		for (int i = 0; i < len; i++) {
			sb.append("header").append(i).append("'s pk=");
			String pk = headers[i].getPath();
			sb.append(pk).append("_");
		}
		return sb.toString();
	}

	/**
	 * ܼҪ
	 * 
	 * @param exts
	 * @return
	 */
	public static String getStringInfo(String[] pks) {
		int len;
		if (ArrayUtils.isEmpty(pks)) {
			len = 0;
		} else {
			len = pks.length;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(len);
		sb.append(".");
		for (int i = 0; i < len; i++) {
			String pk = pks[i];
			sb.append(pk).append("_");
		}
		return sb.toString();
	}

	/**
	 * 
	 * @return ȡļе
	 */
	public static String getDataConfigValue(String key) {
		DataConfiguration dataConfig = FileStorageConfigFactory.getConfig();
		return dataConfig.getString(key);
	}

	/**
	 * ļΪaeʱļ
	 * 
	 * @param file
	 */
	public static File createBakFile(File file) {
		if (!file.exists()) {
			return null;
		} else {
			String parent = file.getParent();
			String fileName = file.getName();
			String fileType = getFileType(fileName);
			String bakFileName;
			// ԭļ ûк׺ֱӼʱļ׺
			if (StringUtils.isEmpty(fileType)) {
				bakFileName = parent + fileName + "." + BAK_FLAG;
			} else {
				int index = fileName.lastIndexOf(".");
				String prefixName = fileName.substring(0, index);
				bakFileName = parent + prefixName + "." + BAK_FLAG;
			}
			File bakFile = new File(bakFileName);
			return bakFile;
		}

	}

}
